/**
 * [ Protocol Manager]
 * @filename ProtocolManager.js
 * @date 18 FEB 2015 </br> </br>
 * @author rishu.yadav
 * @version 1.9
 *
 * Copyright 2015 by Samsung Electronics, Inc.,
 *
 * This software is the confidential and proprietary information of Samsung
 * Electronics, Inc. ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Samsung.
 */

var isCancelTask = false;
var isEnabled = false;
var isFirmwareProgress = false;

var mProtocolManager = {

	/**************************** TO START DEVICE DISCOVERY *****************************
	 *  Call startDeviceDiscovery() method to start device discovery. it will return
	 *  a list of discovered device in JSON format with details.
	 /********************************* ************* *************************************/

	startDeviceDiscovery : function() {
		console.log("@ startDeviceDiscovery");
		var success = function(result) {
			try {
				if (result == NO_DEVICE_FOUND) {
					console.log("no device found: " + result);
					noDeviceFound();
				} else {
					console.log("json list: " + result);
					showDeviceListInNetwork(result);
				}
			} catch(e) {
			}
			mProtocolManager.stopAutoDiscoveryService();
		};
		var error = function(message) {
			console.log("Error: " + message);
			mProtocolManager.stopAutoDiscoveryService();
		};
		mNetworkManager.startAutoDiscService(success, error);
	},

	/**************************** TO STOP DEVICE DISCOPVERY *********************************
	 *  Call stopAutoDiscoveryService() method to stop device discovery.
	 /********************************* ************* ****************************************/

	stopAutoDiscoveryService : function() {
		console.log("@ stopAutoDiscoveryService");
		var success = function(result) {
			console.log("success: " + result);
		};
		var error = function(message) {
			console.log("error: " + message);
		};
		mNetworkManager.stopAutoDiscSrvc(success, error);
	},

	/**************************** TO STOP CONNECTION SERVICE **************************
	 Call stopConnectionService() method to stop connection service
	 with disconnect from device.
	 /********************************* ************* **********************************/

	stopConnectionService : function() {
		console.log("@ stopConnectionService()");
		var success = function(result) {
			console.log("stop connection service success: " + result);
		};
		var error = function(message) {
			console.log("stop connection service fail: " + message);
		};
		mNetworkManager.stopNetworkService(success, error);
	},

	/**************************** TO STOP UPLOADING ***********************************
	 *  Call performCancelOperation() method to stop uploading on LFD
	 /********************************* ************* **********************************/

	performCancelOperation : function(mMaxIndex, mDeviceId, action) {
		console.log("@ performCancelOperation");
		var success = function(result) {
			console.log("cancel upload: " + result);
			if (result == CANCLE_EXECUTING_TASK_SUCCESS){
				updateGUIOnCancel(mMaxIndex, mDeviceId, action);
				var allFlag = 0;
                for (var index = 0; index < mLFDInfoArrayList.length; index++){
                    if(mLFDInfoArrayList[index].mStatus == CANCEL || mLFDInfoArrayList[index].mStatus == FAIL || mLFDInfoArrayList[index].mStatus == DONE)
                    {
                        continue;
                    }else{
                        allFlag = 1;
                    }
                }
                if(allFlag == 0 )
                {
                    if(action == CONNECTION_TYPE_SCHEDULE_UPLOAD)
                    {
                        allUploadFinish();
                    }
                    else if(action == CONNECTION_TYPE_ON_OFF_TIMER_UPDATE)
                    {
                        onOffTimerUpdateCompleted();
                    }
                }
			}
		};
		var error = function(message) {
			console.log("Cancel error message: " + message);
		};
		mNetworkManager.cancelExcutingTask(success, error, action);
	},

	/**************************** INIT NETWORK CALL **********************************
	 * initNetworkConnection method used to initialized network connection for
	 * onOffTimerupdate,express upload & update firmware.
	 /**************************** ************* **************************************/

	//
	initNetworkConnection : function(mNetworkConnectionType) {
		console.log("@initNetworkConnection: " + mNetworkConnectionType);
		isCancelTask = false;
		var mMaxIndex = mLFDInfoArrayList.length - 1;
		mProtocolManager.prepareNetworkConnection(mMaxIndex, mNetworkConnectionType);
	},

	prepareNetworkConnection : function(mMaxIndex, mNetworkConnectionType) {
		console.log("@prepareNetworkConnection: " + mMaxIndex);
		var ConnectSuccess = function(result) {
			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, mNetworkConnectionType);
			} else {
				// In case of no cancel
				if (result == DEVICE_CONNECTION_SUCCESS) {
					setTimeout(function() {
						mLFDInfoArrayList[mMaxIndex].mStatus = CONNECTED;
						excuteNetActionOnConnection(mMaxIndex, mNetworkConnectionType);
					}, 500);
				} else if (result == DEVICE_CONNECTION_FAIL) {
					console.log("!!  Failed to connect with device.  !!");
					if (mNetworkConnectionType == CONNECTION_TYPE_SCHEDULE_UPLOAD) {
						showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SCHEDULE);
					} else {
						// showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_DATA_TO_SIGNAGE);
						showOkPopup(langObj.Settings.MIA_SID_DEVCEI_TRUNED_OFF_NOT_CONNECT_WIRELESS);
					}
					mLFDInfoArrayList[mMaxIndex].mStatus = FAIL;
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, mNetworkConnectionType);
					}, 500);
				} else if (result == CONNECTED_WITH_OTHERS) {
					console.log("!!  Device connected with others.  !!");
					showOkPopup(langObj.Schedule.MIA_SID_ANOTHER_DEVICE_HAS_BEEN_CONNECTED_TO_THE_SIGNAGE_PLEASE_TRY_AGAIN_AFTER_DISCONNECTING);
					mLFDInfoArrayList[mMaxIndex].mStatus = FAIL;
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, mNetworkConnectionType);
					}, 500);
				} else if(result == WRONG_PIN_CODE){

				     //Reset previous pin
				     Utils.setMacPinValue(mLFDInfoArrayList[mMaxIndex].mMacAddr,"");
				     updatePinInDeviceList(mMaxIndex);
                     showOkPopup(langObj.Send.MIA_SID_20_MIX_INVALID_ENTER_PIN_FOR_AGAIN.replace("<<A>>",mLFDInfoArrayList[mMaxIndex].mDeviceName));
                     mLFDInfoArrayList[mMaxIndex].mStatus = FAIL;
                     setTimeout(function() {
                        tryingForNextDevice(mMaxIndex, mNetworkConnectionType);
                     }, 500);
                } else if(result == DEVICE_BLOCKED || result == DEVICE_STATUS_BLOCKED){
                    showOkPopup(langObj.Send.MIA_SID_AUTHOR_MIX_LOCKED_PIN_5_TIMES_MSG.replace(/<<A>>/ig, mLFDInfoArrayList[mMaxIndex].mDeviceName).replace("<<B>>","5"));
                    mLFDInfoArrayList[mMaxIndex].mStatus = FAIL;
                    setTimeout(function() {
                      tryingForNextDevice(mMaxIndex, mNetworkConnectionType);
                    }, 500);
				}
			}
		};
		var ConnectError = function(message) {
			console.log("Connect Error : " + message);
			setTimeout(function() {
				tryingForNextDevice(mMaxIndex, mNetworkConnectionType);
			}, 500);
		};
        var decryptionSuccess = function(result){
            var mLFDInfoLoc = mLFDInfoArrayList[mMaxIndex];
            var deviceIP = mLFDInfoLoc.mDeviceIP;
            var tlsReq = mLFDInfoLoc.mTLSRequired;
            deviceIP = deviceIP.trim();
            var connInfo = deviceIP+">>>>"+tlsReq+">>>>"+result;
            mNetworkManager.manualConnect(ConnectSuccess, ConnectError, connInfo);
        };
        var decryptionError = function(message){
         console.log("@decryptionError: " + message);
        };

		var DisconnectSuccess = function(result) {

			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, mNetworkConnectionType);
			} else {
				if (result == DEVICE_DISCONNECTION_SUCCESS) {
					console.log("!!  Device disconnection success.  !!");
					setTimeout(function() {
					var mLFDInfoLoc = mLFDInfoArrayList[mMaxIndex];
					mLFDInfoLoc.mStatus = DISCONNTED;
					var data = mLFDInfoLoc.mPin;
					var tlsReq = mLFDInfoLoc.mTLSRequired;
					if(tlsReq == '1'){
					    mUtilityManager.doDecryption(decryptionSuccess, decryptionError, data);
					}else{
                        var deviceIP = mLFDInfoLoc.mDeviceIP;
                        deviceIP = deviceIP.trim();
                        var connInfo = deviceIP+">>>>"+tlsReq+">>>>"+data;
                        mNetworkManager.manualConnect(ConnectSuccess, ConnectError, connInfo);

					}
						}, 3500);
				}
			}
		};
		var DisconnectError = function(message) {
			console.log("@DisconnectError: " + message);
		};
		mNetworkManager.stopNetworkService(DisconnectSuccess, DisconnectError);
	},

	/**************************** POWER ON/OFF TIMER UPDATE **********************************
	 * startExpressOnOffTimerUpdate method used to update on-off timer with express update.
	 /**************************** ************* **********************************************/

	startExpressOnOffTimerUpdate : function(mMaxIndex) {
		var onOffTimerSuccess = function(result) {

			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_ON_OFF_TIMER_UPDATE);
			} else {
				switch(result) {
				case ON_OFF_TIMER_UPDATE_SUCCESS:
					console.log("!!  On Off timer set successfully.  !!");
					updateExportProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId, 100);
					setTimeout(function() {
						onOffTimerUpdateSuccess(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
						mMaxIndex = mMaxIndex - 1;
						if (mMaxIndex >= 0) {
							setTimeout(function() {
								mProtocolManager.prepareNetworkConnection(mMaxIndex, CONNECTION_TYPE_ON_OFF_TIMER_UPDATE);
							}, 1000);
						} else {
							onOffTimerUpdateCompleted();

						}
					}, 1000);
					break;
				case ON_OFF_TIMER_UPDATE_FAIL:
					console.log("!!  The power On-Off timer has been failed to update.  !!");
					mLFDInfoArrayList[mMaxIndex].mStatus = FAIL;
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_ON_OFF_TIMER_UPDATE);
					}, 500);
					break;
				case CLOCK_SET_FAIL:
					console.log("!!  The power On-Off timer clock set failed.  !!");
					mLFDInfoArrayList[mMaxIndex].mStatus = FAIL;
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_ON_OFF_TIMER_UPDATE);
					}, 500);
					break;
				case TIMER_SET_FAIL:
					console.log("!!  The power On-Off timer, timer set failed.  !!");
					mLFDInfoArrayList[mMaxIndex].mStatus = FAIL;
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_ON_OFF_TIMER_UPDATE);
					}, 500);
					break;
				case HOLIDAY_SET_FAIL:
					console.log("!!  The power On-Off timer holiday set failed.  !!");
					mLFDInfoArrayList[mMaxIndex].mStatus = FAIL;
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_ON_OFF_TIMER_UPDATE);
					}, 500);
					break;
				default:
					console.log("!!  default case.  !!");
				}
			}
		};
		var onOffTimerFail = function(message) {
			console.log("@onOffTimerFail: " + message);
		};

		mNetworkManager.updateOnOffTimerInfo(onOffTimerSuccess, onOffTimerFail, mLFDInfoArrayList[mMaxIndex].mMetaInfo);
	},

	/*************************  START EXPRESS SCHEDULE UPLOAD  *****************************
	 * To start schedule express upload one by one by invoking startExpressUpload
	 /**************************** ************* ********************************************/

	startExpressUpload : function(mMaxIndex) {
		var ExpressUploadSuccess = function(result) {
			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
			} else {
				switch(result) {
				case UPLOAD_FILES_SUCCESS:
					uploadCompleteSuccess(mMaxIndex);
					prepareConnection(mMaxIndex);
					/*mMaxIndex = mMaxIndex - 1;
					while(mMaxIndex >= 0){
                        if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
                            mMaxIndex = mMaxIndex - 1;
                        }else{
                            break;
                        }
                    }
					if (mMaxIndex >= 0) {
                        setTimeout(function() {
                            mProtocolManager.prepareNetworkConnection(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
                        }, 2000);
					} else {
						// EXPRESS UPLOAD DONE FOR ALL DEVICE
						allUploadFinish();
					}*/
					break;
				case UPLOAD_FILES_FAIL:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SCHEDULE);
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
					}, 500);
					break;
				case INCONSISTENT_UPLOAD_INFO:
					showOkPopup(langObj.Send.MIA_SID_UNABLE_TO_FIND_FILE_SCHEDULE);
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
					}, 500);
					break;
				case DISK_SPACE_ERROR:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SCHEDULE_STORAGE_FULL);
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
					}, 500);
					break;
				case INTERNAL_MEMORY_FULL:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SCHEDULE_STORAGE_FULL);
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
					}, 500);
					break;
				case USB_MEMORY_FULL:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SCHEDULE_USB_FULL);
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
					}, 500);
					break;
				case SDCARD_MEMORY_FULL:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SCHEDULE);
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
					}, 500);
					break;
				case USB_NOT_CONNECTED:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SCHEDULE);
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
					}, 500);
					break;
				case USB_REMOVED_FROM_DEVICE:
                    showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SCHEDULE_DISCONNECTED);
					setTimeout(function() {
						tryingForNextDevice(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
					}, 500);
					break;
				default:
					if (!isNaN(result) && getParseIntInDec(result) <= 100) {
						console.log("ExportProgressValue : %" + result);
						updateExportProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId, result);
					}
				}
			}
		};
		var ExpressUploadError = function(message) {
			console.log("@ExpressUploadError: " + message);
		};

		initProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId);
		mNetworkManager.startUploading(ExpressUploadSuccess, ExpressUploadError, mLFDInfoArrayList[mMaxIndex].mMetaInfo);
	},

	/**************************** FIRMWARE DOWNLOAD & UPLOAD *****************************
	 *  PrepareFirmwareDownload()used to get firmware info & initiate
	 *  firmware download & upload.
	 /**************************** ************* ******************************************/
	// LFD firmwareInfo
	prepareFirmwareDownload : function(mMaxIndex) {
		var getFirmwareInfoSuccess = function(result) {
			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_FIRMWARE_UPGRADE);
			} else {
				if (result == GET_FIRMWARE_INFO_FAIL) {
					showOkPopup(langObj.Schedule.failedFrmWareInfoId);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
				} else {
					if (result != undefined && result != "") {
						try {
							//T-GFSLE2AKUC-0363;T-GFSLEWWS1-0009;;; - For RHE Model
                            //RMNOD>>>>T>>>>T-GFSLE2AKUC-0363;T-GFSLEWWS1-0009;10FDR;XXXXX;- For RMD Model
                            //RH48E>>>>U>>>>T-GSRHEDEUC-0480.0;T-GSRHEDEUS-0018;;;010FER;;-- For RHE Model
                            //QM49H>>>>100>>>>T-KTMLAKUC-0703.10;T-KTMLAKUS-1006;T-NVTLFFWWS-1006;T-YYYYYYYY-1006;010FHQ;010SHQ -TIZEN S4
                            //PM32F>>>>94>>>>T-HKMLAKUC-0719.20;T-HKMLFWWS-1003;T-NVTLFFWWS-1022;T-YYYYYYYY-1003;010FFP;010SFP - TIZEN S5
							var mDeviceInfo = new Array();
							mDeviceInfo = result.split(">>>>");
							//T-GFSLE2AKUC-0363;T-GFSLEWWS1-0009;10FDR;XXXXX;
							var mFirmInfo = mDeviceInfo[2];
							var mSoftwareInfo = new Array();
							mSoftwareInfo = mFirmInfo.split(";");
							var model_code_with_ver = mSoftwareInfo[0];
							//PRODID=010FDR
                            var product_id = "";
                            var model_code_with_ver_arr = new Array();
                            model_code_with_ver_arr = model_code_with_ver.split("-");
                            //PROTYP=T-NT14LLAKUC
                            var model_code = model_code_with_ver_arr[0] + '-' + model_code_with_ver_arr[1];
                            model_code = model_code.trim();
                            // FOR RHE Model Code: [T-GSRHEAKUC - US & GSRHEDEUC - EU] & ProductId:[010FER]
                            if (RHE_MODEL_CODE_US == model_code || RHE_MODEL_CODE_EU == model_code) {
                                //PRODID=010FER
                                product_id = RHE_PRODUCT_CODE;
                            }else if(TIZEN_S4_MODEL_CODE == model_code || TIZEN_S5_MODEL_CODE == model_code || TIZEN_S6_MODEL_CODE == model_code){
                                 // TIZEN S4/S5
                                 product_id = mSoftwareInfo[4];
                            }else{
                              // NT 14
                                product_id = mSoftwareInfo[2];
                            }
                            console.log("******* PRODUCT_CODE:" + product_id + " *****");

							//VERSION=0520.0
							var software_ver = model_code_with_ver_arr[2];
							var firmwareInfo = software_ver + ">>>>" + product_id + ">>>>" + model_code;
							mProtocolManager.stopConnectionService();
							setTimeout(function() {
								mProtocolManager.getSLMServerFirmwareInfo(mMaxIndex, firmwareInfo);
							}, 2500);
						} catch(e) {
							firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
						}
					}
				}
			}
		};
		var getFirmwareInfoError = function(message) {
			firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
			console.log("Error: " + message);
		};
		mNetworkManager.getFirmwareInfo(getFirmwareInfoSuccess, getFirmwareInfoError);
	},

	// SERVER Info
	getSLMServerFirmwareInfo : function(mMaxIndex, mFirmwareInfo) {
		console.log("@getSLMServerFirmwareInfo");
		var srvFirmwareVersion = "";
		var firmwareInfo = new Array();
		firmwareInfo = mFirmwareInfo.split(">>>>");

		var success = function(result) {
			console.log("successInfo: " + result);
			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_FIRMWARE_UPGRADE);
			} else {

				switch(result) {
				case FIRMWARE_ALREADY_DOWNLOADED:
					// In case of firmware already downloaded
					isFirmwareProgress = true;
					genericPopUpObj.createPopUp("Notification", "Type1");
					genericPopUpObj.appendHTMLData([langObj.Schedule.MIA_SID_UPDATE_AVAIL_SIGNAGE + "<br>" + (langObj.Schedule.MIA_SID_MIX_DEVICE_NAME.replace('<<A>>', mLFDInfoArrayList[mMaxIndex].mDeviceName)) + "<br>" + (langObj.Schedule.COM_EBD_SID_MIX_CURRENT_VERSION.replace('<<A>>', firmwareInfo[0])) + "<br>" + (langObj.Schedule.COM_EBD_SID_MIX_LATEST_VERSION.replace('<<A>>', srvFirmwareVersion)) + '']);
					genericPopUpObj.addFooterBtns([{
						"btntxt" : langObj.Settings.COM_SID_CANCEL,
						"callback" : function() {
							showAnimationLoader(langObj.Settings.MIA_SID_CANCELING_DOT);
							isFirmwareProgress = false;
							mLFDInfoArrayList[mMaxIndex].isCanceled = true;
							cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_FIRMWARE_UPGRADE);
							closeGenericPopUp();
						}
					}, {
						"btntxt" : langObj.Settings.COM_SID_UPDATE,
						"callback" : function() {
							closeGenericPopUp();
							genericPopUpObj.createPopUp("Notification", "Type1");
							genericPopUpObj.appendHTMLData([langObj.Settings.MIA_SID_SWITCH_ANOTHER_APP_STOP_UPDATE]);
							genericPopUpObj.addFooterBtns([{
								"btntxt" : COM_SID_OK,
								"callback" : function() {
									$('.list>li#' + deviceID).find('.storageInfo').text(langObj.Settings.MIA_SID_COPYING_DOT);
									$('.list>li#' + deviceID).find('.function_icon').removeClass('del_icon').addClass('cancel_checkbox');
									isFirmwareProgress = true;
									mProtocolManager.performFirmwareUpload(mMaxIndex, mFirmwareInfo);
									closeGenericPopUp();
								}
							}]);
							genericPopUpObj.showPopUp();
						}
					}]);
					genericPopUpObj.showPopUp();
					break;
				case FIRMWARE_DOWNLOAD_FAIL:
					// In case of network issue
					showOkPopup(langObj.Schedule.MIA_FAILED_CONNECT_TO_NETWORK_INTERNET_CONNECTION_UNSTABLE);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case FIRMWARE_UPDATE_NOT_FOUND:
					// in case of error found
					showOkPopup(langObj.Settings.MIA_SID_UNABLE_CHECK_SW_VERSION_SERVER + " " + langObj.Settings.COM_TV_SID_PLEASE_TRY_AGAIN_LATER_KR_CC);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case NO_NEW_FIRMWARE_UPDATE_AVAILABLE:
					// in case of firmware  up to date
					showOkPopup(langObj.Settings.MIA_SID_SIGNAGE_SYSTEM_UP_DATE + "<br>" + (langObj.Schedule.MIA_SID_MIX_DEVICE_NAME.replace('<<A>>', mLFDInfoArrayList[mMaxIndex].mDeviceName)) + "<br>" + (langObj.Schedule.COM_EBD_SID_MIX_CURRENT_VERSION.replace('<<A>>', firmwareInfo[0])) + '');
					cancelFirmwareUpgrade(mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case NEW_FIRMWARE_UPDATE_FOUND:
					closeGenericPopUp();
					// Show firmware info to upload
					genericPopUpObj.createPopUp("Notification", "Type1");
					genericPopUpObj.appendHTMLData([langObj.Schedule.MIA_SID_UPDATE_AVAIL_SIGNAGE + "<br>" + (langObj.Schedule.MIA_SID_MIX_DEVICE_NAME.replace('<<A>>', mLFDInfoArrayList[mMaxIndex].mDeviceName)) + "<br>" + (langObj.Schedule.COM_EBD_SID_MIX_CURRENT_VERSION.replace('<<A>>', firmwareInfo[0])) + "<br>" + (langObj.Schedule.COM_EBD_SID_MIX_LATEST_VERSION.replace('<<A>>', srvFirmwareVersion)) + '']);
					genericPopUpObj.addFooterBtns([{
						"btntxt" : langObj.Settings.COM_SID_CANCEL,
						"callback" : function() {
							showAnimationLoader(langObj.Settings.MIA_SID_CANCELING_DOT);
							isFirmwareProgress = false;
							mLFDInfoArrayList[mMaxIndex].isCanceled = true;
							cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_FIRMWARE_UPGRADE);
							closeGenericPopUp();
						}
					}, {
						"btntxt" : langObj.Settings.COM_SID_UPDATE,
						"callback" : function() {
							closeGenericPopUp();
							genericPopUpObj.createPopUp("Notification", "Type1");
							genericPopUpObj.appendHTMLData([langObj.Settings.MIA_SID_SWITCH_ANOTHER_APP_STOP_UPDATE]);
							genericPopUpObj.addFooterBtns([{
								"btntxt" : COM_SID_OK,
								"callback" : function() {

									$('.list>li#' + deviceID).find('.storageInfo').text(langObj.Settings.COM_TV_SID_DOWNLOADING_DOT_KR_BLANKDOT);
									$('.list>li#' + deviceID).find('.function_icon').removeClass('del_icon').addClass('cancel_checkbox');
									$('.list>li#' + deviceID).removeClass('send-failed');

									isFirmwareProgress = true;
									mProtocolManager.performFirmwareDownload(mMaxIndex, mFirmwareInfo);
									closeGenericPopUp();
								}
							}]);
							genericPopUpObj.showPopUp();
						}
					}]);
					genericPopUpObj.showPopUp();
					break;
				default:
					//alert("srvFirmwareVersion: version: "+srvFirmwareVersion);
					srvFirmwareVersion = result;
				}
			}
		};
		var error = function(message) {
			alert("Error in start Discovered");
			console.log("Error: " + message);
		};

		mNetworkManager.getSLMServerFirmwareVersion(success, error, mFirmwareInfo);
	},

	performFirmwareDownload : function(mMaxIndex, mFirmwareInfo) {
		console.log("@performFirmwareDownload");
		var success = function(result) {
			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_FIRMWARE_UPGRADE);
			} else {
				console.log("Success: " + result);
				switch(result) {
				case FIRMWARE_DOWNLOAD_SUCCESS:
					$('.list>li#' + mLFDInfoArrayList[mMaxIndex].mDeviceId).find('.storageInfo').text(langObj.Settings.MIA_SID_COPYING_DOT);
					mProtocolManager.performFirmwareUpload(mMaxIndex, mFirmwareInfo);
					break;
				case FIRMWARE_DOWNLOAD_FAIL:
					showOkPopup(langObj.Settings.MIA_SID_UNABLE_DOWNLOAD_UPDATE_FILE + " " + langObj.Settings.MIA_SID_WIRELESS_ROUTER_NOT_WORKING_OFF);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case MEMORY_LOW_IN_MOBILE_CLIENT:
					showOkPopup(langObj.Settings.MIA_SID_NOT_ENOUGH_SPACE_SW_UPDATE);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				default :
					console.log("Progress: %" + result);
					var resultArr = new Array();
					resultArr = result.split(">>>>");
					firmwareUpdateProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId, resultArr[0], resultArr[1]);
				}
			}
		};

		var error = function(message) {
			console.log("Error: " + message);
		};

		initProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId);
		mNetworkManager.startFirmwareDownload(success, error);
	},
	performFirmwareUpload : function(mMaxIndex, mFirmwareInfo) {
		console.log("@ startFirmwareUpload");
		var success = function(result) {
			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_FIRMWARE_UPGRADE);
			} else {
				console.log("@ upload result: " + result);

				switch(result) {
				case UPLOAD_FILES_SUCCESS:
					console.log("upload success");
					updateExportProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId, 0);
					firmwareUpdateSuccess(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case INCONSISTENT_UPLOAD_INFO:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SW_UPDATE_SIGNAGE);
					console.log("upload failed");
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case DISK_SPACE_ERROR:
					console.log("space not avaiable on LFD");
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SW_UPDATE_SIGNAGE);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case INTERNAL_MEMORY_FULL:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SW_UPDATE_SIGNAGE);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case USB_MEMORY_FULL:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SW_UPDATE_SIGNAGE);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case SDCARD_MEMORY_FULL:
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SW_UPDATE_SIGNAGE);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case USB_NOT_CONNECTED:
					console.log("USB not connected");
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SW_UPDATE_SIGNAGE);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case USB_REMOVED_FROM_DEVICE:
					console.log("USB removed");
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SW_UPDATE_SIGNAGE);
                    firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
					break;
				case UPLOAD_FILES_FAIL:
                        console.log("Upload Fails");
                        showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_SW_UPDATE_SIGNAGE);
                         firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
                         break;
				default:
					console.log("Progress: %" + result);
					var resultArr = new Array();
					resultArr = result.split(">>>>");
					firmwareCopyProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId, resultArr[0], resultArr[1]);
				}
			}
		};
		var error = function(message) {
			console.log("performUpload Error :" + message);
		};

		var ConnectSuccess = function(result) {
			if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
				// In case of cancel
				cancelExcutionTask(mMaxIndex, CONNECTION_TYPE_FIRMWARE_UPGRADE);
			} else {
				if (result == DEVICE_CONNECTION_SUCCESS) {
					initProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId);
					mNetworkManager.startFirmwareUpload(success, error);
				} else if (result == DEVICE_CONNECTION_FAIL) {
					showOkPopup(langObj.Schedule.MIA_SID_UNABLE_SEND_DATA_TO_SIGNAGE);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
				} else if (result == CONNECTED_WITH_OTHERS) {
					showOkPopup(langObj.Schedule.MIA_SID_ANOTHER_DEVICE_HAS_BEEN_CONNECTED_TO_THE_SIGNAGE_PLEASE_TRY_AGAIN_AFTER_DISCONNECTING);
					firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
				} else if(result == WRONG_PIN_CODE){
                     showOkPopup(langObj.Send.MIA_SID_20_MIX_INVALID_ENTER_PIN_FOR_AGAIN.replace("<<A>>", mLFDInfoArrayList[mMaxIndex].mDeviceName));
                     firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
                } else if(result == DEVICE_BLOCKED || result == DEVICE_STATUS_BLOCKED){
                    showOkPopup(langObj.Send.MIA_SID_AUTHOR_MIX_LOCKED_PIN_5_TIMES_MSG.replace(/<<A>>/ig, mLFDInfoArrayList[mMaxIndex].mDeviceName).replace("<<B>>","5"));
                    firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
				}
			}
		};
		var ConnectError = function(message) {
			console.log("Connect Error : " + message);
		};
		var conInfo = mLFDInfoArrayList[mMaxIndex].mDeviceIP+">>>>"+mLFDInfoArrayList[mMaxIndex].mTLSRequired +">>>>"+mLFDInfoArrayList[mMaxIndex].mPin;
		mNetworkManager.manualConnect(ConnectSuccess, ConnectError, conInfo);
	},
	registerWiFiStateChangeReceiver : function() {
		console.log("@ registerWiFiStateChangeReceiver");
		var success = function(result) {
			if (deviceType == "iPad" || deviceType == "iPhone") {
				updateWiFiState(result);
			} else {
				if (!isEnabled) {
					isEnabled = true;
					updateWiFiState(result);
				}
				setTimeout(function() {
					isEnabled = false;
				}, 2000);
			}
		};
		var error = function(message) {

		};
		mUtilityManager.registerStateChangeReceiver(success, error);
	},
	performIpVerification : function(deviceIP,tls,pin) {
		console.log("@ performManualIpVerification");
		var mConnectionInfoList = new Array();
		var mConnectionInfo = "";
		var success = function(result) {
		    console.log("performIpVerification: response -> "+result);
		    setTimeout(function(){
			if (result == DEVICE_CONNECTION_SUCCESS) {

				if (mConnectionInfo != "") {
					try {
						mConnectionInfoList = mConnectionInfo.split(">>>>");
						var mDeviceName = mConnectionInfoList[0];
						var mDeviceType = mConnectionInfoList[1];
						var mStorageType = mConnectionInfoList[2];
						var mTotalSpace = mConnectionInfoList[3];
						var mAvailableSpace = mConnectionInfoList[4];
						var mVersion = mConnectionInfoList[5];

						var mPlayerId = Utils.getPlayerTypeWithModelCode(mDeviceType);
						var mActivePlayer = Utils.getActivePlayerType();
						mActivePlayer = parseInt(mActivePlayer);
						mPlayerId = parseInt(mPlayerId);
						mStorageType = Utils.convertStorageType(mStorageType);
						if (mPlayerId >= mActivePlayer && mAvailableSpace != undefined && mTotalSpace != undefined && mVersion.length > 0) {
							// Supported
							console.log("Device IP: " + deviceIP + ",Device Name: " + mDeviceName, ",storage Type: " + mStorageType + ",total space: " + mTotalSpace + ",Avaiable space: " + mAvailableSpace + ",Version: " + mVersion);
							addManuallyDevice(STATE_SUPPORTED, STATE_SUPPORTED, 'true', deviceIP, mDeviceName, mStorageType, mTotalSpace, mAvailableSpace, mVersion, mPlayerId);

						} else if (mPlayerId >= mActivePlayer) {
							addManuallyDevice(STATE_UPDATE_REQ, STATE_UPDATE_REQ, 'true', deviceIP, mDeviceName, mStorageType, mTotalSpace, mAvailableSpace, mVersion, mPlayerId);
						} else {
							var unSupported = Utils.getUnSupportedPlayerType();
							addManuallyDevice(STATE_UNSUPPORTED, STATE_UNSUPPORTED, 'true', deviceIP, mDeviceName, mStorageType, mTotalSpace, mAvailableSpace, mVersion, unSupported);
						}
					} catch(e) {
						console.log("Exception: " + e);
					}
				}
                } else if (result == DEVICE_CONNECTION_FAIL || result == DEVICE_BLOCKED || result == WRONG_PIN_CODE) {
                     console.log("DEVICE_CONNECTION_FAIL");
				addManuallyDevice(STATE_FAIL_CONNECTION, STATE_FAIL_CONNECTION, 'false', deviceIP, deviceIP, '', '', '', '', Utils.getActivePlayerType());
			} else if (result == CONNECTED_WITH_OTHERS) {
                     console.log("CONNECTED_WITH_OTHERS");
				addManuallyDevice(STATE_FAIL_CONNECTION, STATE_FAIL_CONNECTION, 'false', deviceIP, deviceIP, '', '', '', '', Utils.getActivePlayerType());
			} else {
                     console.log("RESULT: "+result);
				mConnectionInfo = result;
			}
		    },1000);
		};
		var error = function(message) {
			console.log("@message: " + message);
			addManuallyDevice(STATE_FAIL_CONNECTION, STATE_FAIL_CONNECTION, 'false', deviceIP, deviceIP, '', '', '', '');
		};
		var conInfo = deviceIP+">>>>"+tls+">>>>"+pin;
		//alert("conInfo: "+conInfo);
		mNetworkManager.verifyManualyAddedIp(success, error, conInfo);
	},

	verifyPINInfo : function(ip,resultType,pin,listRefId,isManual) {
    		console.log("@ verifyPINInfo :  " + pin);
    		 var mConnectionInfo = "";
    		 var success = function(result) {
    		    console.log("verifyPINInfo: response -> "+result);
                if (result == DEVICE_CONNECTION_SUCCESS) {
                    setTimeout(function() {
                     updateVerifyPINResult(mConnectionInfo,resultType,listRefId,pin,isManual);
                    }, 1000);
                } else if (result == DEVICE_CONNECTION_FAIL) {
                      loaderObj.hideLoader()
                      mProtocolManager.stopConnectionService();
                      if(isManual == 'true'){
                        addManuallyDevice(STATE_FAIL_CONNECTION, STATE_FAIL_CONNECTION, '0', ip, ip, '', '', '', '', Utils.getActivePlayerType());
                      }else{
                         showOkPopup(langObj.Settings.MIA_SID_DEVCEI_TRUNED_OFF_NOT_CONNECT_WIRELESS);
                      }
                } else if (result == CONNECTED_WITH_OTHERS) {
                        loaderObj.hideLoader()
                        if(isManual == 'true'){
                          addManuallyDevice(STATE_FAIL_CONNECTION, STATE_FAIL_CONNECTION, '0', ip, ip, '', '', '', '', Utils.getActivePlayerType());
                        }else{
                         showOkPopup(langObj.Schedule.MIA_SID_ANOTHER_DEVICE_HAS_BEEN_CONNECTED_TO_THE_SIGNAGE_PLEASE_TRY_AGAIN_AFTER_DISCONNECTING);
                        }
                 } else if ( result == DEVICE_STATUS_BLOCKED || result == DEVICE_BLOCKED) {
                      loaderObj.hideLoader()
                      if(isManual == 'true'){
                       showOkPopup(langObj.Send.MIA_SID_AUTHOR_MIX_LOCKED_PIN_5_TIMES_MSG.replace(/<<A>>/ig, ip).replace("<<B>>","5"));
                       addManuallyDevice(STATE_FAIL_CONNECTION, STATE_FAIL_CONNECTION, '2', ip, ip, '', '', '', '', Utils.getActivePlayerType());
                      }else{
                         var devName =  listRefId.parent('.auto-LFD-List').attr('deviceName');
                         if(devName==undefined){
                             devName =  listRefId.attr('deviceName');
                         }
                       showOkPopup(langObj.Send.MIA_SID_AUTHOR_MIX_LOCKED_PIN_5_TIMES_MSG.replace(/<<A>>/ig, devName).replace("<<B>>","5"));
                       updatecheckBoxBlock(listRefId);
                      }
                 } else if(result == WRONG_PIN_CODE){
                      loaderObj.hideLoader()
                      wrongPinGuiUpdate(mConnectionInfo,resultType,listRefId,pin,isManual);
                 }else {
                    mConnectionInfo = result;
                 }
    		};
    		var error = function(message) {
    			console.log("@message: " + message);
    		};
    		//TLS required true - 1
    		var conInfo = ip + ">>>>1>>>>" +pin;
    		mNetworkManager.verifyPIN(success, error,conInfo);
    	},

    	performHandshaking : function(ip) {
            console.log("@ performHandshaking :  " + ip);
            var success = function(result) {
            console.log("performHandshaking: response -> "+result);
                  if(result == HANDSHAKE_SUCCESS){
                        // open for PIN page
                        // ip -> device IP
                        // true -> add signage TV
                        console.log("HANDSHAKE_SUCCESS");
                        loaderObj.hideLoader();
                        var pinStr = (langObj.Send.MIA_SID_20_MIX_ENTER_PIN_NUMBER_FOR.replace("<<A>>", ip) + " " + langObj.Send.MIA_SID_20_PROCESS_REQUIRED_CONNECTING_TARGET_FIRST);
                        askToEnterCorrectPIN(ip, true, pinStr);
                  }else {
                      // try non TLS Connection
                      //ip -> device ip
                      //"0" -> Non TLS
                      // "" -> empty pin
                      loaderObj.hideLoader();
                      console.log("HANDSHAKE_FAIL");
                      mProtocolManager.performIpVerification(ip,0,"");
                  }
            };
            var error = function(message) {
                console.log("@message: " + message);
            };
            mNetworkManager.startHandshaking(success, error,ip);
	},

	/**************************** RSITE DOWNLOAD TASK ***********************************
	 *  downloadRSiteDesignElements()used to download zip from R Site
	 *  & extract on specific location
	 /**************************** ************* ******************************************/
	downloadRSiteDesignElements : function(fileType, mUrl, mPath, deviceId) {
		console.log("@ downloadRSiteDesignElements");
		var success = function(result) {
			switch(result) {
			case RSITE_DOWNLOAD_SUCCESS:
				downloadSuccess(deviceId);
				break;
			case RSITE_DOWNLOAD_FAIL:
				downloadFailed(deviceId);
				showOkPopup(langObj.Settings.MIA_SID_MIX_UNABLE_DOWNLOAD_FILE.replace("<<A>>", fileType));
				break;
			case WI_FI_TURNED_OFF:
				var showWifiSetUpPopUp = true;

				downloadFailed(deviceId);
				if (showWifiSetUpPopUp) {
					genericPopUpObj.createPopUp("Notification", "Type1", "Wi-Fi");
					genericPopUpObj.appendHTMLData([langObj.Settings.MIA_SID_MIX_UNABLE_DOWNLOAD_FILE.replace("<<A>>", fileType) + "<br/>" + langObj.Settings.MIA_SID_WIFI_TURNED_OFF]);
					genericPopUpObj.addFooterBtns([{
						"btntxt" : langObj.Settings.MIA_SID_SET_UP_WI_FI,
						"callback" : function() {
							closeGenericPopUp();
							mUtilityManager.openWiFiSettings();
						}
					}, {
						"btntxt" : COM_SID_OK,
						"callback" : function() {
							closeGenericPopUp();
						}
					}]);
					genericPopUpObj.showPopUp();
				} else {
					showOkPopup(langObj.Settings.MIA_SID_MIX_UNABLE_DOWNLOAD_FILE.replace("<<A>>", fileType) + "<br/>" + langObj.Settings.MIA_SID_WIFI_TURNED_OFF);
				}
				break;
			case DOWNLOAD_UNZIP_SUCCESS:
				loaderObj.hideLoader();
				updateExportProgressBar(deviceId, 0);
				downloadUnzipped(deviceId);
				break;
			case DOWNLOAD_UNZIP_FAIL:
				downloadFailed(deviceId);
				showOkPopup(langObj.Schedule.MIA_FAILED_CONNECT_TO_NETWORK_INTERNET_CONNECTION_UNSTABLE);
				break;
			case MEMORY_LOW_IN_MOBILE_CLIENT:
				downloadFailed(deviceId);
				showOkPopup(langObj.Settings.MIS_SID_NOT_ENOUGH_SPACE_DOWNLOAD_FILES);
				break;
			default:
				console.log("Progress: %" + result);
				updateExportProgressBar(deviceId, result);
			}
		};
		var error = function(message) {
			console.log("@message: " + message);
		};
		var mMsg = mUrl + ">>>>" + mPath;
		mNetworkManager.downloadRSiteElements(success, error, mMsg);
	},
	deleteRSiteElements : function(mPath, deviceId) {
		console.log("@ deleteRSiteElements");
		var success = function(result) {
			console.log("@Deleted: " + result);
		};
		var error = function(message) {
			console.log("@message: " + message);
		};
		mNetworkManager.deleteRSiteElements(success, error, mPath);
	},
	launchPlayStore : function() {
		console.log("@ launchPlayStore");
		var success = function(result) {
			console.log("@ launchPlayStore: " + result);
		};
		var error = function(message) {
			console.log("@ error: " + message);
		};
		mUtilityManager.openAppStore(success, error);
	},
	doPostHttpRequest : function(mUrl) {
		console.log("@ doPostHttpRequest");
		var success = function(result) {
			console.log("@ doPostHttpRequest: " + result);
		};
		var error = function(message) {
			console.log("@ error: " + message);
		};
		mUtilityManager.doPostRequest(success, error, mUrl);
	},
	requestEncryption : function(refId,isManual,mac,data) {
        console.log("@ requestEncryption");
        var success = function(result) {
           //save encrypted data
           Utils.setMacPinValue(mac,result);
           updateEncryptedInfo(refId,isManual,result);
        };
        var error = function(message) {
            console.log("@ error: " + message);
        };
        mUtilityManager.doEncryption(success, error, data);
    },
	isFileOrFolderExist : function(path, wthrBckGrdObj) {
		console.log("@ isFileOrFolderExists()");
		var success = function(result) {
			if (result) {
				if (Utils.isWeatherImgDownloaded()) {
					console.log("isWeatherImgDownloaded: - true 0 " + Utils.isWeatherImgDownloaded());
					var widgetObject = templateObj.getTextObjectFromId(wthrBckGrdObj.divID).weather;
					setwthrImgIconTxtFun(wthrBckGrdObj.divID, widgetObject.wthrIcon, widgetObject.wthrText, widgetObject.portraitCityImgUrl, widgetObject.landscapeCityImgUrl, templateObj.getTextObjectFromId(wthrBckGrdObj.divID).elementType);
				} else {
					console.log("isWeatherImgDownloaded - false 1: " + Utils.isWeatherImgDownloaded());
					weatherImgDownloadPopup(wthrBckGrdObj);
				}
			} else {
				console.log("isWeatherImgDownloaded - false 2:  " + Utils.isWeatherImgDownloaded());
				weatherImgDownloadPopup(wthrBckGrdObj);
			}
		};
		var error = function(message) {
			console.log("@ error: " + message);
		};
		mUtilityManager.isFileOrDirExist(success, error, path);
	}
};

function excuteNetActionOnConnection(mMaxIndex, action) {
	console.log("@onConnectExcuteNetAction");
	switch(action) {
	case CONNECTION_TYPE_ON_OFF_TIMER_UPDATE:
		initProgressBarTimer(mLFDInfoArrayList[mMaxIndex].mDeviceId);
		updateExportProgressBar(mLFDInfoArrayList[mMaxIndex].mDeviceId, 90);
		setTimeout(function() {
			mProtocolManager.startExpressOnOffTimerUpdate(mMaxIndex);
		}, 500);
		break;
	case CONNECTION_TYPE_SCHEDULE_UPLOAD:
		setTimeout(function() {
			mProtocolManager.startExpressUpload(mMaxIndex);
		}, 500);
		break;
	case CONNECTION_TYPE_FIRMWARE_UPGRADE:
		setTimeout(function() {
			isFirmwareProgress = true;
			mProtocolManager.prepareFirmwareDownload(mMaxIndex);
		}, 500);
		break;
	}
}

function tryingForNextDevice(mMaxIndex, action) {
	console.log("@tryingForNextDevice");
	switch(action) {
	case CONNECTION_TYPE_ON_OFF_TIMER_UPDATE:
		if (mLFDInfoArrayList[mMaxIndex].mStatus != CANCEL) {
			onOffTimerUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
		}
		mMaxIndex = mMaxIndex - 1;
		if (mMaxIndex >= 0) {
			setTimeout(function() {
				mProtocolManager.prepareNetworkConnection(mMaxIndex, CONNECTION_TYPE_ON_OFF_TIMER_UPDATE);
			}, 1000);
		} else {
			// EXPRESS UPLOAD DONE FOR ALL DEVICE
			//Utils.showToast("The power On-Off timer updation completed.");
			onOffTimerUpdateCompleted();
			mProtocolManager.stopConnectionService();
		}
		break;
	case CONNECTION_TYPE_SCHEDULE_UPLOAD:
		if (mLFDInfoArrayList[mMaxIndex].mStatus != CANCEL) {
			uploadFailed(mLFDInfoArrayList[mMaxIndex].mDeviceId, mMaxIndex);
		}
        prepareConnection(mMaxIndex);
		break;
	case CONNECTION_TYPE_FIRMWARE_UPGRADE:
		firmwareUpdateFail(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId);
		break;
	}
}
function prepareConnection(mMaxIndex){
    mMaxIndex = mMaxIndex - 1;
    while(mMaxIndex >= 0){
        if (mLFDInfoArrayList[mMaxIndex].isCanceled) {
            mMaxIndex = mMaxIndex - 1;
        }else{
            break;
        }
    }
    if (mMaxIndex >= 0) {
        setTimeout(function() {
            mProtocolManager.prepareNetworkConnection(mMaxIndex, CONNECTION_TYPE_SCHEDULE_UPLOAD);
        }, 1000);
    } else {
        // EXPRESS UPLOAD DONE FOR ALL DEVICE
        allUploadFinish();
    }
}


function cancelExcutionTask(mMaxIndex, action) {
	console.log("@****** CANCEL EXECUTION TASK *****@");
	// In case of cancel
	switch(action) {
	case CONNECTION_TYPE_ON_OFF_TIMER_UPDATE:
		if (!isCancelTask) {
			isCancelTask = true;
			if (mLFDInfoArrayList[mMaxIndex].mStatus != CANCEL) {
				mProtocolManager.performCancelOperation(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId, action);
			} else {
				tryingForNextDevice(mMaxIndex, action);
				isCancelTask = false;
			}
		}
		break;
	case CONNECTION_TYPE_SCHEDULE_UPLOAD:
		if (!isCancelTask) {
			isCancelTask = true;
			if (mLFDInfoArrayList[mMaxIndex].mStatus != CANCEL) {
				mProtocolManager.performCancelOperation(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId, action);
			} else {
				tryingForNextDevice(mMaxIndex, action);
				isCancelTask = false;
			}
		}
		break;
	case CONNECTION_TYPE_FIRMWARE_UPGRADE:
		if (!isCancelTask) {
			isCancelTask = true;
			mProtocolManager.performCancelOperation(mMaxIndex, mLFDInfoArrayList[mMaxIndex].mDeviceId, action);
		}
		break;
	}
}

function updateGUIOnCancel(mMaxIndex, mDeviceId, action) {

	switch(action) {
	case CONNECTION_TYPE_SCHEDULE_UPLOAD:
		var mCancelCounter = 0;
		for (var index = 0; index < mLFDInfoArrayList.length; index++) {
			if (mLFDInfoArrayList[index].isCanceled) {
				mLFDInfoArrayList[index].mStatus = CANCEL;
				uploadCancel(mLFDInfoArrayList[index].mDeviceId);
			} else {
				mCancelCounter = mCancelCounter + 1;
			}
		}
		if (mMaxIndex > 0 && mCancelCounter > 0) {
			tryingForNextDevice(mMaxIndex, action);
		} else {
			resetCancelUI(mCancelCounter);
			mProtocolManager.stopConnectionService();
		}
		 isCancelTask = false;
		break;
	case CONNECTION_TYPE_ON_OFF_TIMER_UPDATE:
		var mCancelCounter = 0;
		for (var index = 0; index < mLFDInfoArrayList.length; index++) {
			if (mLFDInfoArrayList[index].isCanceled) {
				mLFDInfoArrayList[index].mStatus = CANCEL;
				onOffTimerCanceled(mLFDInfoArrayList[index].mDeviceId);
			} else {
				mCancelCounter = mCancelCounter + 1;
			}
		}

		if (mMaxIndex > 0 && mCancelCounter > 0) {
			isCancelTask = false;
			tryingForNextDevice(mMaxIndex, action);
		} else {
			resetOnOffCancelUI(mCancelCounter);
			mProtocolManager.stopConnectionService();
		}
		break;

	case CONNECTION_TYPE_FIRMWARE_UPGRADE:
		loaderObj.hideLoader();
		mLFDInfoArrayList[mMaxIndex].mStatus = CANCEL;
		cancelFirmwareUpgrade(mLFDInfoArrayList[mMaxIndex].mDeviceId);
		isCancelTask = false;
		break;
	case CONNECTION_TYPE_RSITE_DOWNLOAD:
		setTimeout(function() {
			loaderObj.hideLoader();
			downloadCancel(mDeviceId);
			isCancelTask = false;
		}, 500);
		break;
	}
}

function showOkPopup(message) {
	setTimeout(function() {
		genericOkPopUP("Type1", 'Notification', COM_SID_OK, [message]);
	}, 500);
}

function updateManuallyAddedIP(mConnectionInfo,deviceIP){

  if (mConnectionInfo != "") {
    try {
        mConnectionInfoList = mConnectionInfo.split(">>>>");
        var mDeviceName = mConnectionInfoList[0];
        var mDeviceType = mConnectionInfoList[1];
        var mStorageType = mConnectionInfoList[2];
        var mTotalSpace = mConnectionInfoList[3];
        var mAvailableSpace = mConnectionInfoList[4];
        var mVersion = mConnectionInfoList[5];

        var mPlayerId = Utils.getPlayerTypeWithModelCode(mDeviceType);
        var mActivePlayer = Utils.getActivePlayerType();
        mActivePlayer = parseInt(mActivePlayer);
        mPlayerId = parseInt(mPlayerId);
        mStorageType = Utils.convertStorageType(mStorageType);
        if (mPlayerId >= mActivePlayer && mAvailableSpace != undefined && mTotalSpace != undefined && mVersion.length > 0) {
            // Supported
            console.log("Device IP: " + deviceIP + ",Device Name: " + mDeviceName, ",storage Type: " + mStorageType + ",total space: " + mTotalSpace + ",Avaiable space: " + mAvailableSpace + ",Version: " + mVersion);
            addManuallyDevice(STATE_SUPPORTED, STATE_SUPPORTED, '1', deviceIP, mDeviceName, mStorageType, mTotalSpace, mAvailableSpace, mVersion, mPlayerId);

        } else if (mPlayerId >= mActivePlayer) {
            addManuallyDevice(STATE_UPDATE_REQ, STATE_UPDATE_REQ, '1', deviceIP, mDeviceName, mStorageType, mTotalSpace, mAvailableSpace, mVersion, mPlayerId);
        } else {
            var unSupported = Utils.getUnSupportedPlayerType();
            addManuallyDevice(STATE_UNSUPPORTED, STATE_UNSUPPORTED, '1', deviceIP, mDeviceName, mStorageType, mTotalSpace, mAvailableSpace, mVersion, unSupported);
        }
    } catch(e) {
        console.log("Exception: " + e);
    }
  }
}

