/**
 * @filename PlaylistGenerator.js
 * @author r.k.pandey
 * @date
 *
 * Copyright 2015 by Samsung Electronics, Inc.,
 *
 * This software is the confidential and proprietary information of Samsung
 * Electronics, Inc. ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Samsung.
 */
var playListGenerator = {
    scheduleFolder : null,
    playListFolder : null,
    playListXML : null,
    isPlaylistSch : false,
    isPlaylistCopy : false,

    // Save XML Data
    savePlayListXMLData : function(targetPlayListFolder, targetScheduleName, isPlaylistFromSch, isCopy) {
		// Deferred used to fix DF170228-00990
		var deferred = $.Deferred();
        this.playListFolder = targetPlayListFolder;
        this.scheduleFolder = targetScheduleName;
        this.isPlaylistSch = isPlaylistFromSch;
        this.isPlaylistCopy = isCopy;
		createScheduleFolder().done(function() {
			deferred.resolve(true);
		}).fail(function(error) {
			deferred.reject(error);
		});
		return deferred.promise();
    }
};

function createScheduleFolder() {
	var deferred = $.Deferred();

	var createScheduleFolderForPlaylist = function(fileSystem) {
    fileSystem.root.getDirectory(playListGenerator.scheduleFolder, {
        create : true,
        exclusive : false
    }, createPlayListFolder, failPlaylistDir);
	};

	var createPlayListFolder = function() {
    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, createPlayList, failPlayList);

	};

	var failScheduleFolder = function(error) {
    console.log(error.code);
		deferred.reject(error);
	};

	var failPlayList = function(error) {
    console.log(error.code);
		deferred.reject(error);
	};

	var createPlayList = function(fileSystem) {
    var playlistFold = playListGenerator.scheduleFolder + "/" + playListGenerator.playListFolder;
    fileSystem.root.getDirectory(playlistFold, {
        create : true,
        exclusive : false
    }, createPlayListFile, failPlaylistDir);

	};

	var createPlayListFile = function() {
    if (playListGenerator.isPlaylistCopy) {
        playListGenerator.playListXML = playListGenerator.scheduleFolder + "/" + playListGenerator.playListFolder + "/" + playListGenerator.playListFolder + ".xml";
    } else {
        playListGenerator.playListXML = playListGenerator.scheduleFolder + "/" + playListGenerator.playListFolder + "/" + playListObj.CustomPlaylist[0].Name + ".xml";
    }
    //alert("playListGenerator.playListXML   "+playListGenerator.playListXML);
    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, createPlayListFS, failPlaylistWriter);
	};

	var createPlayListFS = function(fileSystem) {
    fileSystem.root.getFile(playListGenerator.playListXML, {
        create : true,
        exclusive : false
    }, gotPlayListFileEntry, failWriter);
	};

	var gotPlayListFileEntry = function(fileEntry) {
    fileEntry.createWriter(gotPlayListFileWriter, failWriter);
	};

	var failPlaylistDir = function(error) {
		console.log(error.code);
		deferred.reject(error);
	};
	var failPlaylistWriter = function(error) {
		console.log(error.code);
		deferred.reject(error);
	};
	var failWriter = function(error) {
		console.log(error.code);
		deferred.reject(error);
	};
	var gotPlayListFileWriter = function(writer) {
    var returnType = createPlayListXML();
    //alert("playList  "+playList);
    if (returnType) {
        writer.write(playList);
    }
    if (!playListGenerator.isPlaylistSch) {
        /*setTimeout(function(){
         Utils.showToast("Saved successfully.");
         }, 500);*/
    } else {
        playListGenerator.isPlaylistSch = false;
    }

		deferred.resolve();
	};
	window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, createScheduleFolderForPlaylist, failScheduleFolder);

	return deferred.promise();
}

function createPlayListXML() {
    for (var i = 0; i < playListObj.CustomPlaylist.length; i++) {
        var xmlTag = '<?xml version="1.0" encoding="utf-8"?>';
        var atts = '';
        if (playListGenerator.isPlaylistCopy) {
            var timeStamp = new Date().getTime();
            atts = {
                name : playListGenerator.playListFolder,
                timestamp : "'" + timeStamp + "'"
            };
        } else {
            atts = {
                name : playListObj.CustomPlaylist[0].Name,
                timestamp : "'" + playListObj.CustomPlaylist[0].TimeStamp + "'"
            };
        }

        var contentTag = createContentTag(playListObj.CustomPlaylist[i].CustomContent);
        playList = xmlTag + ScheduleUtils.writeElement('playlist', '' + contentTag, atts);
        return true;
    }

}

function createContentTag(content) {
    var content_Tag = '';
    for (var n = 0; n < content.length; n++) {
        var name = ScheduleUtils.writeElement('name', content[n].Name);
        var duration = ScheduleUtils.writeElement('duration', content[n].Duration);
        var path = ScheduleUtils.writeElement('path', content[n].Path);
        var actualPath = ScheduleUtils.writeElement('actualPath', content[n].ActualPath);
        var templateId = ScheduleUtils.writeElement('templateId', content[n].TemplateId);
        content_Tag = content_Tag + ScheduleUtils.writeElement('content', '' + duration + name + path + actualPath + templateId, "");

    }
    return content_Tag;
}

