/*
 * Created by amber.m 16-11-2016
 * File for keeping trnsform matrix to apply transform on any object
 *
 */

var AffineTransform = function(){
    this.Sx = 1.0;
    this.Sy = 1.0;
    this.Tx = 0.0;
    this.Ty = 0.0;
    this.Theta = 0.0;
    this.bIsInv = false;
    this.TransformMat = [[1,0,0],[0,1,0],[0,0,1]];
};

AffineTransform.prototype.ResetTransform=function(){
    this.Sx = 1.0;
    this.Sy = 1.0;
    this.Tx = 0.0;
    this.Ty = 0.0;
    this.Theta = 0.0;
    this.bIsInv = false;
};

AffineTransform.prototype.SetTransform=function(transform){
    this.Sx = transform.Sx;
    this.Sy = transform.Sy;
    this.Tx = transform.Tx;
    this.Ty = transform.Ty;
    this.Theta = transform.Theta;
};

AffineTransform.prototype.InvertTransform=function(){
    this.bIsInv = true;
};


AffineTransform.prototype.SetScale=function(xScaleFactor, yScaleFactor, bConcat){
    if(bConcat){
        this.Sx *= xScaleFactor;
        this.Sy *= yScaleFactor;
    }else{
        this.Sx = xScaleFactor;
        this.Sy = yScaleFactor;
    }
};

AffineTransform.prototype.SetTranslation=function(xTranslation, yTranslation, bConcat){
    if(bConcat){
        this.Tx *= xTranslation;
        this.Ty *= yTranslation;
    }else{
        this.Tx = xTranslation;
        this.Ty = yTranslation;
    }
};

AffineTransform.prototype.SetRotation=function(rotation, bConcat){
    if(bConcat){
        this.Theta += rotation;
    }else{
        this.Theta += rotation;
    }
};

AffineTransform.prototype.GetTransformMatrix=function(bGenerate){
    var matrix = [[1,0,0],[0,1,0],[0,0,1]];
    if(bGenerate)
    {
        if(m_bIsInverse)
        {
            matrix[0][0] = Math.cos(this.Theta);
            matrix[0][1] = Math.sin(this.Theta);
            matrix[0][2] = -(this.Tx * Math.cos(this.Theta) + this.Ty * Math.sin(this.Theta));
            matrix[1][0] = -Math.sin(this.Theta);
            matrix[1][1] = Math.cos(this.Theta);
            matrix[1][2] = (this.Tx * Math.sin(this.Theta) - this.Ty * Math.cos(this.Theta));
            matrix[2][0] = 0;
            matrix[2][1] = 0;
            matrix[2][2] = 1;
        }
        else
        {
            matrix[0][0] = this.Sx*cos(this.Theta);
            matrix[0][1] = this.Sy * -sin(this.Theta);
            matrix[0][2] = this.Tx;
            matrix[1][0] = this.Sx * sin(this.Theta);
            matrix[1][1] = this.Sy * cos(this.Theta);
            matrix[1][2] = this.Ty;
            matrix[2][0] = 0;
            matrix[2][1] = 0;
            matrix[2][2] = 1;
        }
    }
    else
    {
        matrix[0][0] = this.TransformMat[0][0];
        matrix[0][1] = this.TransformMat[0][1];
        matrix[0][2] = this.TransformMat[0][2];
        matrix[1][0] = this.TransformMat[1][0];
        matrix[1][1] = this.TransformMat[1][1];
        matrix[1][2] = this.TransformMat[1][2];
        matrix[2][0] = this.TransformMat[2][0];
        matrix[2][1] = this.TransformMat[2][1];
        matrix[2][2] = this.TransformMat[2][2];
    }
    return matrix;
};

AffineTransform.prototype.TransformPoint=function(point){
    var originalPt = {x:0,y:0};
    originalPt.x = point.x;
    originalPt.y = point.y;
    point.x = originalPt.x*m_dTransformMatrix[0][0] + originalPt.y*m_dTransformMatrix[0][1] + m_dTransformMatrix[0][2];
    point.y = originalPt.x*m_dTransformMatrix[1][0] + originalPt.y*m_dTransformMatrix[1][1] + m_dTransformMatrix[1][2];
    return point;
};
