/*
 * File Detail: All template slideshow effect implentation
 * Created by amber.m 19-01-2015
 * Modified By
 * amber.m 24-01-2015 ::  Added effects SHORTFADE (East West North South), CUT_BLACK_FADE, WIPE, CURTAIN, CORNOR
 * amber.m 27-01-2015 ::  Added effects ZOOM_UP and UNCOVER_3D, Added basic functionality for QUAD transition effects & fixed transition stop issues
 * amber.m 7-02-2015 :: Effect repeat once related issues fixed 
 * amber.m 10.2.2015 :: Wipe north south and west effects added
 * veer.singh 9.2.2016:: optimized some part of code and update quadrant effect.
 */
//Image Effect Type Constants
var TEffectType = {
	NONE:109,
    CUT_BLACK_FADE: 111,
    SHORTFADE_EAST: 112,
    SHORTFADE_WEST: 113,
    SHORTFADE_NORTH: 114,
    SHORTFADE_SOUTH: 115,
    WIPE_EAST: 116,
    WIPE_WEST: 117,
    WIPE_NORTH: 118,
    WIPE_SOUTH: 119,       
    CURTAIN: 120,
    QUADRANT: 121,
    ZOOM_UP: 122,
    SWISS: 123,
    UNCOVER_3D: 124
};
var TEffectRepeatType = {
    INFINITE: 201,
    ONCE: 202
};
var TEffectFrameLayout = {
    SINGLE:301,
    QUAD:302,
    SWISS:303
};
var transitionEffect = function(){
    this.effectType = TEffectType.RANDOM;
    this.frameInfo = null;
    this.repeatDuration = 5;//5secs
    this.effectTimer = 0;
    this.effectTimeout="";
    this.repeatType = TEffectRepeatType.INFINITE;
    this.eFrameLayout = TEffectFrameLayout.SINGLE;
    this.timeOut=10;
    this.animTimeStr=" 1.33s ease-in-out";
};

transitionEffect.prototype.setEffectType = function(eType){
    this.effectType = eType;
};

transitionEffect.prototype.setFrameInfo = function(frmInfo){
    this.frameInfo = frmInfo;
};

transitionEffect.prototype.setRepeatDuration = function(tDuration){
    this.repeatDuration = tDuration;
};

transitionEffect.prototype.setRepeatType = function(repeat){
    this.repeatType = repeat;
};

transitionEffect.prototype.startTransition = function(){
    if(this.frameInfo == null){
        log.debug("ERROR::FRAME INFO FOR EFFECT IS NULL");
        return false;
    }
    if(this.frameInfo.element.length == 0){
        log.debug("ERROR::FRAME INFO ELEMENTS COUNT IS ZERO");
        return false;
    }
    if(this.repeatDuration < 5){
        this.repeatDuration = 5;
    }    
    this.resetTransition();
    log.debug("START1","startTransition");
    if(this.effectType != TEffectType.QUADRANT && this.effectType != TEffectType.SWISS){
        if(this.frameInfo==null)
        {
            return;
        }
        var curObj = this;
        var eleCount = this.frameInfo.element.length;
        var frame = this.frameInfo;
        var curIndex = 0;
        var curNorVisDivId = frame.id+"_ED1";
        var curEffDivId = frame.id+"_ED2";
        
        this.doBasicSettings(frame);
        if(eleCount == 1){
            this.setImageInEDiv(frame, curIndex, frame.id+"_ED1");
            this.setImageInEDiv(frame, curIndex, frame.id+"_ED2");
        }else if(eleCount > 1){
            this.setImageInEDiv(frame, curIndex, frame.id+"_ED1");
            curIndex++;
            this.setImageInEDiv(frame, curIndex, frame.id+"_ED2");
        }else{
            return false;
        }
        switch(this.effectType){
            case TEffectType.CUT_BLACK_FADE: {
                this.applyCutBlackFade(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.SHORTFADE_EAST:
            case TEffectType.SHORTFADE_WEST:
            case TEffectType.SHORTFADE_NORTH:
            case TEffectType.SHORTFADE_SOUTH:
            {
                this.applyShortFade(this.effectType, curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.WIPE_EAST:
            {
                this.applyWipeEast(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.WIPE_WEST:
            {
                this.applyWipeWest(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.WIPE_NORTH:
            {
                this.applyWipeNorth(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.WIPE_SOUTH:       
            {
                this.applyWipeSouth(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.CURTAIN:
            {
                this.applyCurtain(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.CORNOR:
            {
                this.applyCornor(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.ZOOM_UP:
            {
                this.applyZoomUp(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            case TEffectType.UNCOVER_3D:
            {
                this.applyUncover3D(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId);
                break;
            }
            default:
            break;            
        }
    }else {
        switch(this.effectType){
        case TEffectType.SWISS: {
                this.startSwissTransition();
                break;
            }
        case TEffectType.QUADRANT: 
                this.startQuadTransition();
            break;
        default:            
            break;        
        }        
    }
};
transitionEffect.prototype.startQuadTransition=function(){
    if(this.frameInfo==null)
    {
        return;
    }
    var curObj = this;
    var eleCount = this.frameInfo.element.length;
    var frame = this.frameInfo;
    var curIndex = 0;
    var curNorVisSubDivId = "ED1";
    var curEffSubDivId = "ED2";
    
    this.eFrameLayout = TEffectFrameLayout.QUAD;
    if(eleCount==0)
    {
        return false;
    }    
    this.doQuadBasicSettings(frame);
    if(eleCount == 1){
        this.setImageInQuadEDiv(frame, curIndex, "ED1");
        this.setImageInQuadEDiv(frame, curIndex, "ED2");
    }else if(eleCount > 1){
        this.setImageInQuadEDiv(frame, curIndex, "ED1");
        curIndex++;
        this.setImageInQuadEDiv(frame, curIndex, "ED2");
    }
    for(var index = 1;index <= 4;index++){
        $("#"+frame.id+"_"+curNorVisSubDivId+"_"+index).css({'z-index':'101'});                
    }   
    this.applyQuadrant(curObj, eleCount, frame, curIndex, curNorVisSubDivId, curEffSubDivId);              
};
transitionEffect.prototype.startSwissTransition=function(){
    if(this.frameInfo==null)
    {
        return;
    }
    var curObj = this;
    var eleCount = this.frameInfo.element.length;
    var frame = this.frameInfo;
    var curIndex = 0;
    var curNorVisSubDivId = "ED1";
    var curEffSubDivId = "ED2";
    if(eleCount==0)
    {
        return false;
    }
    this.eFrameLayout = TEffectFrameLayout.SWISS;
    this.doSwissBasicSettings(frame);
    if(eleCount == 1){
        this.setImageInSwissEDiv(frame, curIndex, "ED1");
        this.setImageInSwissEDiv(frame, curIndex, "ED2");
    }else if(eleCount > 1){
        this.setImageInSwissEDiv(frame, curIndex, "ED1");
        curIndex++;
        this.setImageInSwissEDiv(frame, curIndex, "ED2");
    }
    for(var index = 1;index <= 4;index++){
        $("#"+frame.id+"_"+curNorVisSubDivId+"_"+index).css({'z-index':'101'});
    }
    this.applySwiss(curObj, eleCount, frame, curIndex, curNorVisSubDivId, curEffSubDivId);    
};
transitionEffect.prototype.resetTransition = function(){
    $(".tpl_EffectsDivCls1,.tpl_EffectsDivCls,.tpl_EffectsImgCls").remove();
    if(this.frameInfo != null)
    {
       $("#"+this.frameInfo.id+"_Img").show();
    }
    clearTimeout(this.effectTimeout);      
    clearInterval(this.effectTimer);
    this.effectTimer = 0;
    this.effectTimeout="";
};
transitionEffect.prototype.stopTransition = function(){
    this.resetTransition();        
    this.frameInfo = null;   
};

transitionEffect.prototype.doBasicSettings = function(frame){
    $("#"+frame.id+"_Img").hide();    
    $("#"+frame.id).append('<div id="'+frame.id+'_ED1" class="tpl_EffectsDivCls"></div>');
    $("#"+frame.id+"_ED1").append('<img class="tpl_EffectsImgCls" id="' + frame.id + '_ED1_Img"></img>');
    Utils.setDimXYWH(frame.id+"_ED1",0,0,frame.w,frame.h);    
    $("#"+frame.id).append('<div id="'+frame.id+'_ED2" class="tpl_EffectsDivCls"></div>');
    $("#"+frame.id+"_ED2").append('<img class="tpl_EffectsImgCls" id="' + frame.id + '_ED2_Img"></img>');
    Utils.setDimXYWH(frame.id+"_ED2",0,0,frame.w,frame.h);
};
transitionEffect.prototype.doQuadBasicSettings = function(frame){
    var frameID=frame.id;
    $("#"+frame.id+"_Img").hide();      
    var frameH_Hlf=frame.h/2;
    var frameW_Hlf=frame.w/2;
    for(var index = 1;index <= 4;index++){
        $("#"+frameID).append('<div id="'+frameID+'_ED1_'+index+'" class="tpl_EffectsDivCls1"></div><div id="'+frameID+'_ED2_'+index+'" class="tpl_EffectsDivCls1"></div>');
        $("#"+frameID+"_ED1_"+index).css({'width':frameW_Hlf+'px','height':frameH_Hlf+'px'});       
        $("#"+frameID+"_ED2_"+index).css({'width':frameW_Hlf+'px','height':frameH_Hlf+'px'});                
    }
    $("#"+frameID+"_ED1_1,#"+frameID+"_ED2_1").css({'top':'0px','left':'0px'});
    $("#"+frameID+"_ED1_2,#"+frameID+"_ED2_2").css({'top':'0px','right':'0px'});
    $("#"+frameID+"_ED1_3,#"+frameID+"_ED2_3").css({'left':'0px','bottom':'0px'});
    $("#"+frameID+"_ED1_4,#"+frameID+"_ED2_4").css({'right':'0px','bottom':'0px'});
};
transitionEffect.prototype.doSwissBasicSettings = function(frame){
   $("#"+frame.id+"_Img").hide();    
    for(var index = 1;index <= 4;index++){
        $("#"+frame.id).append('<div id="'+frame.id+'_ED1_'+index+'" class="tpl_EffectsDivCls1"></div>');
        $("#"+frame.id+"_ED1_"+index).append('<img class="tpl_EffectsImgCls" id="' + frame.id + '_ED1_'+index+'_Img"></img>');        
    }
    var hlfW=frame.w/2;
    var hlfH=frame.h/2;
    Utils.setDimXYWH(frame.id+"_ED1_1",0,0,hlfW,hlfH);
    Utils.setDimXYWH(frame.id+"_ED1_2",hlfW,0,hlfW,hlfH);
    Utils.setDimXYWH(frame.id+"_ED1_3",0,hlfH,hlfW,hlfH);
    Utils.setDimXYWH(frame.id+"_ED1_4",hlfW,hlfH,hlfW,hlfH); 
    for(var index = 1;index <= 4;index++){
        $("#"+frame.id).append('<div id="'+frame.id+'_ED2_'+index+'" class="tpl_EffectsDivCls1"></div>');
        $("#"+frame.id+"_ED2_"+index).append('<img class="tpl_EffectsImgCls" id="' + frame.id + '_ED2_'+index+'_Img"></img>');        
    }
    Utils.setDimXYWH(frame.id+"_ED2_1",0,0,hlfW,hlfH);
    Utils.setDimXYWH(frame.id+"_ED2_2",hlfW,0,hlfW,hlfH);
    Utils.setDimXYWH(frame.id+"_ED2_3",0,hlfH,hlfW,hlfH);
    Utils.setDimXYWH(frame.id+"_ED2_4",hlfW,hlfH,hlfW,hlfH);    
};

transitionEffect.prototype.setImageInEDiv = function(frame, eleIndex, divId){
    var curObj = this;
    var elm=frame.element[eleIndex];
    var imgSrc = elm.imageSrc;    
    var img = document.getElementById(divId+"_Img");
	img.src = imgSrc;	        
	curObj.calculateImgPosInFrame(eleIndex, divId+"_Img");
	Utils.setDimXYWH(divId+"_Img",elm.x,elm.y,elm.w,elm.h);
};
transitionEffect.prototype.setImageInQuadEDiv = function(frame, eleIndex, subId){
    var curObj = this;
    var divId = frame.id;    
    var elm=frame.element[eleIndex];
    var imgSrc = elm.imageSrc;
    curObj.calculateImgPosInFrame(eleIndex, divId);
    var bgLeft,bgTop;  
    var frameW_Hlf=frame.w/2;
    var frameH_Hlf=frame.h/2;
    
    for(var i=1;i<=4;i++)
    {
       $("#"+divId+"_"+subId+"_"+i).css({
        'background-image':"url('"+imgSrc+"')",
        'background-size':elm.w+'px '+elm.h+'px'        
       }); 
    }
    bgLeft=elm.x;
    bgTop=elm.y;
    $("#"+divId+"_"+subId+"_1").css({                
        'background-position': bgLeft+'px '+bgTop+'px'      
    });
    bgLeft=elm.x-frameW_Hlf;
    bgTop=elm.y;
    $("#"+divId+"_"+subId+"_2").css({                
        'background-position': bgLeft+'px '+bgTop+'px'      
    });
    bgLeft=elm.x;
    bgTop=elm.y-frameH_Hlf;
    $("#"+divId+"_"+subId+"_3").css({                
        'background-position': bgLeft+'px '+bgTop+'px'      
    });
    bgLeft=elm.x-frameW_Hlf;
    bgTop=elm.y-frameH_Hlf;
    $("#"+divId+"_"+subId+"_4").css({                
        'background-position': bgLeft+'px '+bgTop+'px'      
    });
};
transitionEffect.prototype.setImageInSwissEDiv = function(frame, eleIndex, subId){
    var curObj = this;
    var divId = frame.id;
    var elm=frame.element[eleIndex];
    var imgSrc = elm.imageSrc;    
    for(var i=1;i<=4;i++)
    {
        $("#"+divId+"_"+subId+"_"+i+"_Img").attr("src",imgSrc);
    }   
	curObj.calculateImgPosInFrame(eleIndex, divId+"_Img");	
	Utils.setDimXYWH(divId+"_"+subId+"_1_Img",elm.x,elm.y,elm.w,elm.h);
	Utils.setDimXYWH(divId+"_"+subId+"_2_Img",-elm.w/2,elm.y,elm.w,elm.h);
	Utils.setDimXYWH(divId+"_"+subId+"_3_Img",elm.x,-elm.h/2,elm.w,elm.h);
	Utils.setDimXYWH(divId+"_"+subId+"_4_Img",-elm.w/2,-elm.h/2,elm.w,elm.h);    
};
transitionEffect.prototype.applyCutBlackFade = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
        'z-index':'101',
    });
    var tFunct = function(){
        $("#"+curEffDivId).css({
           'z-index':'101',
           'opacity':0,
           'transition':'all 0 ease 0',
           'display':'block'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            $("#"+curNorVisDivId).css({
                'display':'none'
            });
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv).css({
                'opacity':1,
                'transition':'opacity 1.5s ease-in-out'
            });            

        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applyShortFade = function(eType, curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
    	'z-index':'101',
    });

    var tFunct = function(){
        var marginStr = curObj.getMarginStrSFEffect();
        $("#"+curEffDivId).css({
            'z-index':'101',
            'opacity':0,
            'margin':marginStr,
            'transition':'all 0 ease 0'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100',
            'margin':'0px 0px 0px 0px'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
           curEffDivId = curNorVisDivId;
           curNorVisDivId = effectDiv;
            $("#"+effectDiv).css({
                'margin':'0px 0px 0px 0px',
                'opacity':1,
                'transition':'margin 1s cubic-bezier(.5,0.25,0.15,0.1), opacity 1s ease-in'
            });
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 500);
            }else{
               curIndex++;
               if(curIndex >= eleCount){
                   curIndex = 0;
               }
               curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1000);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applyWipeEast = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
        'z-index':'101',
    });

    var tFunct = function(){
        $("#"+curEffDivId).css({
            'z-index':'101',
            'transition':'all 0 ease 0',
            'width':'0px'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv).css({
                'width':frame.w+'px',
                'transition':'width'+curObj.animTimeStr
            });            

        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applyWipeWest = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
        'z-index':'101',
    });
    
    var tFunct = function(){
        $("#"+curEffDivId).css({
            'z-index':'101',
            'transition':'all 0 ease 0',
            'width':'0px',
            'margin-left':frame.w+'px'
        });
        $("#"+curEffDivId+"_Img").css({
              'left':-(frame.w-frame.element[curIndex].x)+'px'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv+"_Img").css({
                'left':frame.element[curIndex].x+'px',
                'transition':'left'+curObj.animTimeStr
            });
            $("#"+effectDiv).css({
                'width':frame.w+'px',
                'margin-left':'0px',
                'transition':'width'+curObj.animTimeStr+', margin-left'+curObj.animTimeStr
            });
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applyWipeNorth = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
        'z-index':'101',
    });
    
    var tFunct = function(){
        $("#"+curEffDivId).css({
            'z-index':'101',
            'transition':'all 0 ease 0',
            'height':'0px',
            'margin-top':frame.h+'px'
        });
        $("#"+curEffDivId+"_Img").css({
            'top':-(frame.h-frame.element[curIndex].y)+'px'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv+"_Img").css({
                'top':frame.element[curIndex].y+'px',
                'transition':'top'+curObj.animTimeStr
            });
            $("#"+effectDiv).css({
                'height':frame.h+'px',
                'margin-top':'0px',
                'transition':'height'+curObj.animTimeStr+', margin-top'+curObj.animTimeStr
            });
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};
transitionEffect.prototype.applyWipeSouth = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
        'z-index':'101',
    });
    
    var tFunct = function(){
        $("#"+curEffDivId).css({
            'z-index':'101',
            'transition':'all 0 ease 0',
            'height':'0px'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv).css({
                'height':frame.h+'px',
                'transition':'height'+curObj.animTimeStr
            });
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applyCurtain = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
        'z-index':'101',
    });

    var tFunct = function(){
        
        $("#"+curEffDivId).css({
            'z-index':'101',
            'transition':'all 0 ease 0',
            'width':'0px',
            'margin-left':frame.w/2+'px'
        });
        $("#"+curEffDivId+"_Img").css({
            'left':-(frame.element[curIndex].w/2)+'px'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv+"_Img").css({
                'left':frame.element[curIndex].x+'px',
                'transition':'left'+curObj.animTimeStr
            });
            $("#"+effectDiv).css({
                'width':frame.w+'px',
                'margin-left':'0px',
                'transition':'margin-left'+curObj.animTimeStr+',width'+curObj.animTimeStr
            });
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applyCornor = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
        'z-index':'101',
    });

    var tFunct = function(){
        $("#"+curEffDivId).css({
            'z-index':'101',
            'transition':'all 0 ease 0',
            'width':'0px',
            'height':'0px'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv).css({
                'width':frame.w+'px',
                'height':frame.h+'px',
                'transition':'width'+curObj.animTimeStr+', height'+curObj.animTimeStr
            });
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applyZoomUp = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
    'z-index':'101',
    });

    var tFunct = function(){

        $("#"+curEffDivId).css({
            'z-index':'101',
            'transition':'all 0 ease 0',
            'width':'0px',
            'height':'0px',
            'margin-left':frame.w/2+'px',
            'margin-top':frame.h/2+'px'
        });
        $("#"+curEffDivId+"_Img").css({
            'left':-(frame.element[curIndex].w/2)+'px',
            'top':-(frame.element[curIndex].h/2)+'px'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv+"_Img").css({
                'left':frame.element[curIndex].x+'px',
                'top':frame.element[curIndex].y+'px',
                'transition':'left 1s ease-in-out,top 1s ease-in-out'
            });
            $("#"+effectDiv).css({
                'width':frame.w+'px',
                'height':frame.h+'px',
                'margin-left':'0px',
                'margin-top':'0px',
                'transition':'margin-left 1s ease-in-out,width 1s ease-in-out,margin-top 1s ease-in-out,height 1s ease-in-out'
            });
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applyUncover3D = function(curObj, eleCount, frame, curIndex, curNorVisDivId, curEffDivId){
    $("#"+curNorVisDivId).css({
    'z-index':'101'
    });


    var tFunct = function(){
        $("#"+curEffDivId).css({
            'transition':'all 0 ease 0',
            'z-index':'101',
            'transform':'rotateY(90deg)'
        });
        $("#"+curNorVisDivId).css({
            'z-index':'100'
        });
        var effectDiv = curEffDivId;
        var norDiv = curNorVisDivId;        
        setTimeout(function(){
            curEffDivId = curNorVisDivId;
            curNorVisDivId = effectDiv;
            $("#"+effectDiv).css({
                'transition':'all 1s ease-in-out',
                'transform':'rotateY(0deg)'
            });
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.applySwiss = function(curObj, eleCount, frame, curIndex, curNorVisSubDivId, curEffSubDivId){
    var tFunct = function(){

        $("#"+frame.id+"_"+curEffSubDivId+"_1").css({'z-index':'101','transition':'all 0 ease 0','width':'0px','height':'0px'});
        $("#"+frame.id+"_"+curEffSubDivId+"_2").css({'z-index':'101','transition':'all 0 ease 0','width':'0px','height':'0px','left':frame.w+'px'});
        $("#"+frame.id+"_"+curEffSubDivId+"_2_Img").css({'left':-(frame.element[curIndex].w + frame.element[curIndex].x)+'px'});
        $("#"+frame.id+"_"+curEffSubDivId+"_3").css({'z-index':'101','transition':'all 0 ease 0','width':'0px','height':'0px','top':frame.h+'px'});
        $("#"+frame.id+"_"+curEffSubDivId+"_3_Img").css({'top':-(frame.element[curIndex].h+frame.element[curIndex].y)+'px'});
        $("#"+frame.id+"_"+curEffSubDivId+"_4").css({'z-index':'101','transition':'all 0 ease 0','width':'0px','height':'0px','left':frame.w+'px','top':frame.h+'px'});
        $("#"+frame.id+"_"+curEffSubDivId+"_4_Img").css({'top':-(frame.element[curIndex].h+frame.element[curIndex].y)+'px','left':-(frame.element[curIndex].w + frame.element[curIndex].x)+'px'});
        
        for(var index = 1;index <= 4;index++){
            $("#"+frame.id+"_"+curNorVisSubDivId+"_"+index).css({'z-index':'100'});
        }
        var effectDiv = curEffSubDivId;
        var norDiv = curNorVisSubDivId;        
        setTimeout(function(){
            curEffSubDivId = curNorVisSubDivId;
            curNorVisSubDivId = effectDiv;
            $("#"+frame.id+"_"+effectDiv+"_1").css({'width':frame.w/2+'px','height':frame.h/2+'px','transition':'width'+curObj.animTimeStr+', height'+curObj.animTimeStr});
            $("#"+frame.id+"_"+effectDiv+"_2").css({'width':frame.w/2+'px','height':frame.h/2+'px', 'left':frame.w/2+'px','transition':'width'+curObj.animTimeStr+', height'+curObj.animTimeStr+', left'+curObj.animTimeStr});
            $("#"+frame.id+"_"+effectDiv+"_2_Img").css({'left':-frame.element[curIndex].w/2+'px', 'transition':'left'+curObj.animTimeStr});
            $("#"+frame.id+"_"+effectDiv+"_3").css({'width':frame.w/2+'px','height':frame.h/2+'px', 'top':frame.h/2+'px','transition':'width'+curObj.animTimeStr+', height'+curObj.animTimeStr+', top'+curObj.animTimeStr});
            $("#"+frame.id+"_"+effectDiv+"_3_Img").css({'top':-frame.element[curIndex].h/2+'px', 'transition':'top'+curObj.animTimeStr});
            $("#"+frame.id+"_"+effectDiv+"_4").css({'width':frame.w/2+'px','height':frame.h/2+'px', 'left':frame.w/2+'px', 'top':frame.h/2+'px','transition':'width'+curObj.animTimeStr+', height'+curObj.animTimeStr+', top'+curObj.animTimeStr+', left'+curObj.animTimeStr});
            $("#"+frame.id+"_"+effectDiv+"_4_Img").css({'left':-frame.element[curIndex].w/2+'px', 'top':-frame.element[curIndex].h/2+'px', 'transition':'top'+curObj.animTimeStr+',left'+curObj.animTimeStr});
         },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInSwissEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
                                   
};

transitionEffect.prototype.applyQuadrant = function(curObj, eleCount, frame, curIndex, curNorVisSubDivId, curEffSubDivId){
    var tFunct = function(){        
        var elm=frame.element[curIndex];
        var frameWBy4=frame.w/4;
        var frameHBy4=frame.h/4;
        var frameW_Hlf=frame.w/2;
        var frameH_Hlf=frame.h/2;
        var bgLeft,bgTop;
        var index = 1;
        
       for(index = 1;index <= 4;index++){            
            $("#"+frame.id+"_"+curEffSubDivId+"_"+index).css({                  
                'height':'0px'
                ,'width':'0px'
                ,'z-index':'101'
                ,'transition-property':'width ,top ,height ,left ,background-position ,bottom ,right'
                ,'transition-duration':'0s'
                ,'transition-timing-function':'ease-in-out'                                               
            });  
        }
        bgLeft=elm.x-frameWBy4;
        bgTop=elm.y-frameHBy4;                           
        $("#"+frame.id+"_"+curEffSubDivId+"_1").css({'top':frameHBy4+'px','left':frameWBy4+'px','background-position':bgLeft+'px '+bgTop+'px'});
        bgLeft=elm.x-frameW_Hlf-frameWBy4;
        bgTop=elm.y-frameHBy4;  
        $("#"+frame.id+"_"+curEffSubDivId+"_2").css({'top':frameHBy4+'px','right':frameWBy4+'px','background-position':bgLeft+'px '+bgTop+'px'});
        bgLeft=elm.x-frameWBy4;
        bgTop=elm.y-frameH_Hlf-frameHBy4;
        $("#"+frame.id+"_"+curEffSubDivId+"_3").css({'bottom':frameHBy4+'px','left':frameWBy4+'px','background-position':bgLeft+'px '+bgTop+'px'});
        bgLeft=elm.x-frameW_Hlf-frameWBy4;
        bgTop=elm.y-frameH_Hlf-frameHBy4;
        $("#"+frame.id+"_"+curEffSubDivId+"_4").css({'bottom':frameHBy4+'px','right':frameWBy4+'px','background-position':bgLeft+'px '+bgTop+'px'});
    
        for(index = 1;index <= 4;index++){
            $("#"+frame.id+"_"+curNorVisSubDivId+"_"+index).css({'z-index':'100'});
            $("#"+frame.id+"_"+curEffSubDivId+"_"+index).css({                
                'transition-duration':'0.7s'                
            });             
        }
        var effectDiv = curEffSubDivId;
        var norDiv = curNorVisSubDivId;        
        setTimeout(function(){
            curEffSubDivId = curNorVisSubDivId;
            curNorVisSubDivId = effectDiv;
            bgLeft=elm.x;
            bgTop=elm.y;            
            $("#"+frame.id+"_"+effectDiv+"_1").css({'width':frameW_Hlf+'px','height':frameH_Hlf+'px','left':'0px', 'top':'0px','background-position':bgLeft+'px '+bgTop+'px'});
            bgLeft=elm.x-frameW_Hlf;
            bgTop=elm.y-frameH_Hlf;
            $("#"+frame.id+"_"+effectDiv+"_4").css({'width':frameW_Hlf+'px','height':frameH_Hlf+'px', 'right':'0px', 'bottom':'0px','background-position':bgLeft+'px '+bgTop+'px'});
            setTimeout(function(){
                bgLeft=elm.x-frameW_Hlf;
                bgTop=elm.y;            
                $("#"+frame.id+"_"+effectDiv+"_2").css({'width':frameW_Hlf+'px','height':frameH_Hlf+'px', 'right':'0px', 'top':'0px','background-position':bgLeft+'px '+bgTop+'px'});
                bgLeft=elm.x;
                bgTop=elm.y-frameH_Hlf;
                $("#"+frame.id+"_"+effectDiv+"_3").css({'width':frameW_Hlf+'px','height':frameH_Hlf+'px', 'left':'0px', 'bottom':'0px','background-position':bgLeft+'px '+bgTop+'px'});
            
            },400);
        },curObj.timeOut);
        curObj.effectTimeout=setTimeout(function(){            
            if(curObj.repeatType == TEffectRepeatType.ONCE){                
                setTimeout(function(){
                    curObj.stopTransition();
                }, 200);
            }else{
                curIndex++;
                if(curIndex >= eleCount){
                    curIndex = 0;
                }
                curObj.setImageInQuadEDiv(curObj.frameInfo, curIndex, norDiv);
            }
        }, 1500);
    };
    tFunct();
    if(this.repeatType == TEffectRepeatType.INFINITE){
       this.effectTimer = setInterval(tFunct,this.repeatDuration*1000);
    }
};

transitionEffect.prototype.getMarginStrSFEffect = function(){
    var marginStr = "";
    switch(this.effectType){
        case TEffectType.SHORTFADE_EAST: {
            marginStr = "0px 0px 0px "+(-this.frameInfo.w)+"px";//t r b l
            break;
        }
        case TEffectType.SHORTFADE_WEST: {
            marginStr = "0px 0px 0px "+this.frameInfo.w+"px";
            break;
        }
        case TEffectType.SHORTFADE_NORTH: {
            marginStr = this.frameInfo.h+"px 0px 0px 0px";
            break;
        }
        case TEffectType.SHORTFADE_SOUTH: {
            marginStr = -this.frameInfo.h+"px 0px 0px 0px";
            break;
        }
        default:
            log.debug("WRONG EFFECT","getMarginStrSFEffect");
        break;
        
    }
    return marginStr;
};

transitionEffect.prototype.calculateImgPosInFrame=function(eleIndex, imgId)
{
	var frame = this.frameInfo;
	var ele = frame.element[eleIndex];
	if(ele.imageSrc!="")
	{
	    var imgObj=$("#"+imgId);
	    if(ele.orgW==0 || ele.orgH==0)
	    {
	    	ele.orgW = $("#"+imgId).width();
	    	ele.orgH =$("#"+imgId).height();
	    }
	    var calH=ele.orgH,calW=ele.orgW;
	    var wRatio;
		var hRatio;
		var newH,newW;
		var margin=0;
		var rotationFlag=false;
        // alert(ele.rotation);
	    if(ele.rotation==90 || ele.rotation==270)
	    {
	    	hRatio=frame.w/calH;
			wRatio=frame.h/calW;
			if(wRatio > hRatio)
			{
				newW=wRatio*calW;
				newH=wRatio*calH;
				margin=getParseIntInDec((newH-frame.h)/2);
				if(margin<0)
				{
					margin=margin*-1;
				}
				ele.margin.t=-margin;
				ele.margin.b=-margin;
				ele.y=-margin;
			}
			else
			{
				newW=hRatio*calW;
				newH=hRatio*calH;
				margin=getParseIntInDec((newW-frame.w)/2);
				if(margin<0)
				{
					margin=margin*-1;
				}
				ele.margin.l=-margin;
				ele.margin.r=-margin;
				ele.x=-margin;
			}
	    }
	    else
        {
	   		wRatio=frame.w/calW;
			hRatio=frame.h/calH;
			if(wRatio > hRatio)
			{
				newW=wRatio*calW;
				newH=wRatio*calH;
				margin=getParseIntInDec((newH-frame.h)/2);
				ele.margin.t=-margin;
				ele.margin.b=-margin;
				ele.y=-margin;
			}
			else
			{
				newW=hRatio*calW;
				newH=hRatio*calH;
				margin=getParseIntInDec((newW-frame.w)/2);
				ele.margin.l=-margin;
				ele.margin.r=-margin;
				ele.x=-margin;
			}
        }
		ele.w=newW;
		ele.h=newH;        
	}
};