/*
 * Created by amber.m 08-11-2016 for handling group object 
 *
 * Dependant Files : Template.js, Group.js, RotationUtil.js
 */
'use strict';
var GroupHandler = function(){
    this.groupIdCounter = 0;
    this.groupMap ={};
    this.rotationUtil = new RotationUtil();
    this.selGrpElmList = [];
    this.drag=[];
};

/*return root group for group id*/
GroupHandler.prototype.findRoot=function(groupId){
    var groupObj = this.groupMap[groupId];
    if(groupObj.parentId == ""){
        return groupId;
    }
    return this.findRoot(groupObj.parentId);
};

/* find and return the bounding rectangle of a group */
GroupHandler.prototype.boundingRectForGroup=function(group)
{
	var temp = {x:group.x, y:group.y, w:group.w, h:group.h};
	var rect ={minX:Number.MAX_VALUE, 
		minY:Number.MAX_VALUE, 
		maxX:Number.MIN_VALUE, 
		maxY:Number.MIN_VALUE};
  	group.calculateOuterRectForMagneticPointForAllGroupElements(group, rect);
  
  	temp.x = rect.minX;
  	temp.y = rect.minY;
  	temp.h = rect.maxY - rect.minY;
  	temp.w = rect.maxX - rect.minX;
  	
  	return temp;
  	
};
/*iterate recursively into all the sub groups for a given group id and pushes all the elements in the elementArray*/
GroupHandler.prototype.getGroupElements=function(grpId, elementArray)
{
	var group = this.groupMap[grpId];
	var grpElmLen = group.elements.length;
	for (var i = 0; i < grpElmLen; i++) {
		var id =  group.elements[i].id;
		if(TemplateUtils.isElementTypeGroup(id))
		{
			this.getGroupElements(id, elementArray);
		}
		else
		{
			elementArray.push(group.elements[i]);
		}
	}
};

/*Update the element reference objects which includes updating the x,y,w,h,rotation for a selected group whose elements is already populated into selGrpElmList */
GroupHandler.prototype.updateGrpElmObj=function(rotation, copyRefToObj)
{
	var len = this.selGrpElmList.length;
	if(len <= 0)
	{
		log.debug("ERROR len = " + len + " copyRefToObj = " + copyRefToObj,"updateGrpElmObj");
	}
	var isAnyShape = false;
	var shapeObjArr = [];
	for (var i = 0; i < len; i++)
	{
		var element = this.selGrpElmList[i];
		this.updateElementObject(copyRefToObj, element, rotation);
		isAnyShape = this.updateMnFrmElmOnTouchEnd(element, shapeObjArr);
	}
	if (isAnyShape)
    {
        templateObj.renderShapes(shapeObjArr);
    }
};

GroupHandler.prototype.updateElementObject=function(copyRefToObj, element, rotation)
{
	if(copyRefToObj)
	{
		templateObj.updateObjXYHWFromObj(element,element.ref);
		element.ref.rotation = element.ref.rotation + rotation;
		var tempRotation = element.ref.rotation;
		if(element.ref.rotation < 0)
		{
			element.ref.rotation = element.ref.rotation + 360;
		}
		else if(element.ref.rotation > 0)
		{
			element.ref.rotation = element.ref.rotation % 360;
		}
	}
	else
	{
		templateObj.updateObjXYHWFromObj(element.ref, element);
		//templateObj.resetMnFrmElementPos(element.id,element.ref);
		//templateObj.setMainFrmElmAngle(element.id, element.ref.rotation);
		if (TemplateUtils.isElementTypeMainFrame(element.id))
		{
			templateObj.resetMnFrmElementPos(element.id,element.ref);
			templateObj.setMainFrmElmAngle(element.id, element.ref.rotation);
		}
		else
		{
			templateObj.setAngleForFrameHtml(element.id, element.ref.rotation);
			templateObj.updateBlkFrameDimention(element.id, element.ref.x, element.ref.y, element.ref.w, element.ref.h);
		}
	}
};

/*iterates recursively into all the sub groups and update all its elements which includes updating the x,y,w,h,rotation for a given element */
GroupHandler.prototype.multiSelectUpdateGrpElmObj=function(grpId, rotation, copyRefToObj, shapeObjArr)
{
	var group = this.groupMap[grpId];
	var grpElmLen = group.elements.length;
	var isAnyShape = false;
	for (var i = 0; i < grpElmLen; i++) {
		var id =  group.elements[i].id;
		if(TemplateUtils.isElementTypeGroup(id))
		{
			this.multiSelectUpdateGrpElmObj(id, rotation, copyRefToObj);
		}
		else
		{
			var element = group.elements[i];
			var elmID = element.id;
			this.updateElementObject(copyRefToObj, element, rotation);
			if (shapeObjArr != null && shapeObjArr != undefined) {
                isAnyShape = this.updateMnFrmElmOnTouchEnd(element, shapeObjArr);
            }
		}
	}
	/*if (isAnyShape)
    {
        templateObj.renderShapes(shapeObjArr);
    }*/
};

GroupHandler.prototype.updateMnFrmElmOnTouchEnd = function(element, shapeObjArr) {
    var isAnyShape = false;
    var elementType = element.ref.elementType;
    if(TemplateUtils.isElementTypeMainFrame(element.id))
    {
        var fontfamily = templateObj.getTextObjectFromId(element.id).fontFamily;
        var type = templateObj.getElementTypeFromId(element.id);
        if(fontfamily!=undefined && fontfamily!="" && type == TEXT)
        {
            templateObj.textImageUpdate(element.id);
        } else if (TemplateUtils.isShape(elementType)) {
            var currShapeObj = {
                divID: element.id,
                prop: element.ref,
                shapeType: shapeInfoMapping[elementType].Id
            };
            isAnyShape = true;
            shapeObjArr.push(currShapeObj);
        }
    }
    return isAnyShape;
};

/*get new position co-ordinates for an element when a given a given group is rotated by a given angle. The element center is repositioned with respect to group center */
GroupHandler.prototype.getRectForGrpElmRotation=function(groupId, angle, element)
{
	var parentGroup = this.groupMap[groupId];
	var mainFrmOffset=$("#"+FRAME_ID_PREFIX+"Main").offset();
	var groupCenter = 
	{
		x:parentGroup.x + parentGroup.w/2 + mainFrmOffset.left,
		y:parentGroup.y + parentGroup.h/2 + mainFrmOffset.top
	};
	var elementCenterPoint = 
	{
		x:element.x + element.w/2 + mainFrmOffset.left,
		y:element.y + element.h/2 + mainFrmOffset.top
	};
	var newElementCenter = this.rotationUtil.doTransformation(groupCenter, elementCenterPoint,angle, true);
	var rect =
	{
		x:newElementCenter.x - element.w/2 - mainFrmOffset.left,
		y:newElementCenter.y - element.h/2 - mainFrmOffset.top,
		w:element.w,
		h:element.h
	};
	log.debug("rect x = " + rect.x + " y = " + rect.y + " w = " + rect.w + " h = " + rect.h ,"getRectForGrpElmRotation");
	return rect;
};

GroupHandler.prototype.checkBoundaryCondition=function(rect, angle)
{
	var curElmObjPoints ={mainFrameSelElmObj:{x:rect.x,y:rect.y,w:rect.w,h:rect.h,rotation:angle},mainFrame:templateObj.mainFrame};
	var isElmInside = TemplateUtils.isElmInsideMainFrame(curElmObjPoints);
	return isElmInside;
};
/*check whether the selected group elements is getting outside main frame boundary when the group is rotated by an angle. The selGrpElmList is populated in touch start event of group rotation div  */
GroupHandler.prototype.chkGrpElmRotForMnFrmBoundary=function(groupId, angle)
{
	var result = true;
	var len = this.selGrpElmList.length;
	if(len <= 0)
	{
		log.debug("ERROR len  = " + len + " groupId = " + groupId,"chkGrpElmRotForMnFrmBoundary");
	}
	
	for (var i = 0; i < len; i++) 
	{
		var element = this.selGrpElmList[i];
		var rect = this.getRectForGrpElmRotation(groupId, angle, element);
		var totalRotation = this.selGrpElmList[i].ref.rotation + angle;
		result = this.checkBoundaryCondition(rect, totalRotation);
		if(result == false)
		{
			log.debug("ERROR chkGrpElmRotForMnFrmBoundary returns false angle = " + angle + " original ref angle  = " + this.selGrpElmList[i].ref.rotation,"chkGrpElmRotForMnFrmBoundary");
			break;
		}
	}
	return result;
};

/*iterates recursively into all the sub groups and check all its elements whether they are going outside main frame boundary when a group is rotated by an angle*/
GroupHandler.prototype.multiSelectChkGrpElmRotForMnFrmBoundary=function(parentGroupId, groupId, angle)
{
	var group = this.groupMap[groupId];
	var result = true;
	var grpElmLen = group.elements.length;
	for (var i = 0; i < grpElmLen; i++) 
	{
		var id =  group.elements[i].id;
		if(TemplateUtils.isElementTypeGroup(id))
		{
			result = this.multiSelectChkGrpElmRotForMnFrmBoundary(parentGroupId, id,angle);
			if(result == false)
			{
				return result;
			}
		}
		else
		{
			var element = group.elements[i];
			var rect = this.getRectForGrpElmRotation(parentGroupId, angle, element);
			var totalRotation = element.ref.rotation + angle;
			result = this.checkBoundaryCondition(rect, totalRotation);
			if(result == false)
			{
				log.debug("ERROR multiSelectChkGrpElmRotForMnFrmBoundary element.ref.rotation = " + element.ref.rotation + " angle = " + angle,"multiSelectChkGrpElmRotForMnFrmBoundary");
				return result;
			}
		}
	}
	
	return result;
};

/*Rotate all the elements of the selected group by a given angle.The selGrpElmList is populated in touch start event of group rotation div */
GroupHandler.prototype.rotateGroupElm=function(groupId, angle)
{
    for (var i = 0; i < this.selGrpElmList.length; i++)
    {
        var element = this.selGrpElmList[i];
        var rect = this.getRectForGrpElmRotation(groupId, angle, element);
        templateObj.updateObjXYHWFromObj(element.ref, rect);
        if (TemplateUtils.isElementTypeMainFrame(element.id))
        {
            templateObj.setMainFrmElmAngle(element.id, element.ref.rotation + angle);
            templateObj.resetMnFrmElementPos(element.id, element.ref);
        }
        else
        {
            templateObj.setAngleForFrameHtml(element.id, element.ref.rotation + angle);
            templateObj.updateBlkFrameDimention(element.id, element.ref.x, element.ref.y, element.ref.w, element.ref.h);
        }
    }
};

/*iterate recursively and rotate all sub group elements by a given angle for a given group id*/
GroupHandler.prototype.multiSelectRotateGrpElm=function(parentGroupId, groupId, angle)
{
	var group = this.groupMap[groupId];
	var grpElmLen = group.elements.length;
	for (var i = 0; i < grpElmLen; i++) {
		var id =  group.elements[i].id;
		if(TemplateUtils.isElementTypeGroup(id))
		{
			this.multiSelectRotateGrpElm(parentGroupId, id,angle);
		}
		else
		{
			var element = group.elements[i];
			var rect = this.getRectForGrpElmRotation(parentGroupId, angle, element);
			templateObj.updateObjXYHWFromObj(element.ref, rect);
			if (TemplateUtils.isElementTypeMainFrame(element.id))
			{
				templateObj.setMainFrmElmAngle(element.id, element.ref.rotation + angle);
				templateObj.resetMnFrmElementPos(element.id, element.ref);
			}
			else
			{
				templateObj.setAngleForFrameHtml(element.id, element.ref.rotation + angle);
				templateObj.updateBlkFrameDimention(element.id, element.ref.x, element.ref.y, element.ref.w, element.ref.h);
			}
		}
	}
};

/*check whether the selected group elements is getting outside main frame boundary when the group is dragged by deltaX and deltaY .The selGrpElmList is populated in touch start event of group div*/
/*GroupHandler.prototype.chkGrpElmDragForMnFrmBoundary=function(deltaX, deltaY)
{
	var result = true;
	var len = this.selGrpElmList.length;
	for (var i = 0; i < len; i++) 
	{
		var element = this.selGrpElmList[i];
		var rect =
		{
			x:element.ref.x + deltaX,
			y:element.ref.y + deltaY,
			w:element.ref.w,
			h:element.ref.h
		};
		result = this.checkBoundaryCondition(rect, element.ref.rotation);
		if(!result)
		{
			log.debug("ERROR chkGrpElmDragForMnFrmBoundary deltaX = " + deltaX + " deltaY  = " + deltaY + " rotation = " + element.ref.rotation,"chkGrpElmDragForMnFrmBoundary");
			break;
		}
	}
	return result;
};*/
GroupHandler.prototype.isGroupElementMovementValid=function(deltaX, deltaY,isTouchEnd,isFromCP)
{
	var result = true;
	var len = this.selGrpElmList.length;
	for (var i = 0; i < len; i++)
	{
		var element = this.selGrpElmList[i];
		var rect =
		{
			x:element.ref.x,
			y:element.ref.y,
			w:element.ref.w,
			h:element.ref.h
		};
		if(!isTouchEnd)
		{
		    rect.x=rect.x+deltaX;
		    rect.y=rect.y+deltaY;
		}
		result = this.checkBoundaryCondition(rect, element.ref.rotation);
		if(result && isTouchEnd)
		{
		  result=templateObj.isValidMoveForMultiSelectElement(element,isFromCP);
		}
		if(!result)
		{
		    break;
		}
	}
	return result;
};

/*Update all the selected group elements by deltaX and deltaY when the group is dragged by deltaX and deltaY.The selGrpElmList is populated in touch start event of group div*/
GroupHandler.prototype.dragGrpElm=function(deltaX,deltaY)
{
	for (var i = 0; i < this.selGrpElmList.length; i++) 
	{
		var element = this.selGrpElmList[i];
		element.ref.x = element.ref.x + deltaX;
		element.ref.y = element.ref.y + deltaY;
//		templateObj.resetMnFrmElementPos(element.id,element.ref);
		
		if (TemplateUtils.isElementTypeMainFrame(element.id))
		{
			templateObj.resetMnFrmElementPos(element.id, element.ref);
		}
		else
		{
			templateObj.updateBlkFrameDimention(element.id, element.ref.x, element.ref.y, element.ref.w, element.ref.h);
		}
	}
};

/*iterate recursively and check whether the any of the sub group and its elements is getting outside main frame boundary when the group(grpId) is dragged by deltaX and deltaY */
GroupHandler.prototype.multiSelectChkGrpElmDragForMnFrmBoundary=function(grpId, deltaX, deltaY)
{
	var group = this.groupMap[grpId];
	var result = true;
	var grpElmLen = group.elements.length;
	for (var i = 0; i < grpElmLen; i++) {
		var id =  group.elements[i].id;
		if(TemplateUtils.isElementTypeGroup(id))
		{
			result = this.multiSelectChkGrpElmDragForMnFrmBoundary(id, deltaX, deltaY);
			if(!result)
			{
				return result;
			}
		}
		else
		{
			var element = group.elements[i];
			var rect =
			{
				x:element.x + deltaX,
				y:element.y + deltaY,
				w:element.w,
				h:element.h
			};
			result = this.checkBoundaryCondition(rect, element.ref.rotation);
			if(!result)
			{
				log.debug("ERROR multiSelectChkGrpElmDragForMnFrmBoundary element.ref.rotation = " + element.ref.rotation +" deltaX = " + deltaX + " deltaY  = " + deltaY ,"multiSelectChkGrpElmDragForMnFrmBoundary");
				return result;
			}
		}
	}
	return result;
};

/*iterate recursively and update sub group and its elements by deltaX and deltaY when the group is dragged by deltaX and deltaY based on the updateX, updateY flag passed*/
GroupHandler.prototype.multiSelectDragGrpElm=function(grpId, deltaX,deltaY,updateX, updateY)
{
	var group = this.groupMap[grpId];
	var grpElmLen = group.elements.length;
	for (var i = 0; i < grpElmLen; i++) {
		var id =  group.elements[i].id;
		if(TemplateUtils.isElementTypeGroup(id))
		{
			this.multiSelectDragGrpElm(id, deltaX, deltaY, updateX, updateY);
		}
		else
		{
			var element = group.elements[i];
			if(updateX)
			{
				element.ref.x = element.x + deltaX;
			}
			if(updateY)
			{
				element.ref.y = element.y + deltaY;
			}
			if (TemplateUtils.isElementTypeMainFrame(element.id))
			{
				templateObj.resetMnFrmElementPos(element.id, element.ref);
			}
			else
			{
				templateObj.updateBlkFrameDimention(element.id, element.ref.x, element.ref.y, element.ref.w, element.ref.h);
			}
		}
	}
};

/*get new position co-ordinates for group elements when the group is resized using control points, the new rect is group current position co-ordinates and groupObj points to original group co-ordinates before resizing
 The distance between group center and element center gets repositioned/changed with respect to the percentage change in group width and height*/
GroupHandler.prototype.getRectForElmOnGroupResize=function(groupObj, newRect, rotation, element)
{
	var mainFrmOffset=$("#"+FRAME_ID_PREFIX+"Main").offset();
	
	var gx = groupObj.x + mainFrmOffset.left;
	var gy = groupObj.y + mainFrmOffset.top;
	var gw = groupObj.w;
	var gh = groupObj.h;
	
	var centerPoint = {
			x:gx + gw/2, 
			y:gy + gh/2
		};
	
	var x = element.x + mainFrmOffset.left;
	var y = element.y + mainFrmOffset.top;
	var w = element.w;
	var h = element.h;

	var cx = x + w / 2;
	var cy = y + h / 2;
	
	var tobeRotated = {
						x:cx, 
						y:cy
					};
	
	var tp = this.rotationUtil.doTransformation(centerPoint, tobeRotated,-rotation, true);
	
	var dx = tp.x - gx;
	var dy = tp.y - gy;
	var updatedFx = newRect.x + mainFrmOffset.left;
	var updatedFy = newRect.y + mainFrmOffset.top;
	var updatedWidth = newRect.w;
	var updatedHeight = newRect.h;
	var percentChangeInWidth = ((updatedWidth - gw) * 100) / gw;
	var percentChangeInHeight = ((updatedHeight - gh) * 100) / gh;
	var updatedDx = ((percentChangeInWidth * dx) / 100) + dx;
	var updatedDY = ((percentChangeInHeight * dy) / 100) + dy;
	var newCx = updatedFx + updatedDx;
	var newCy = updatedFy + updatedDY;
	
	var newElementCenter = {
				x:newCx, 
				y:newCy
			};
	var newGroupCenter = {
			x:updatedFx + updatedWidth/2, 
			y:updatedFy + updatedHeight/2
			};
	
	var newTransformedElementCenter = this.rotationUtil.doTransformation(newGroupCenter, newElementCenter,rotation, true);
	var newWidth = ((percentChangeInWidth * w) / 100) + w;
	var newHeight = ((percentChangeInHeight * h) / 100) + h;
	var unRotatedX = newTransformedElementCenter.x - newWidth / 2;
	var unRotatedY = newTransformedElementCenter.y - newHeight / 2;
	
	var rect = {
			x:unRotatedX - mainFrmOffset.left,
			y:unRotatedY - mainFrmOffset.top,
			w:newWidth,
			h:newHeight
	};
	
	log.debug("rect.x:"+rect.x + " rect.y = " + rect.y + " rect.w = " + rect.w + " rect.h = " + rect.h, "getRectForElmOnGroupResize");
	return rect;
};

/*On group resize, proportionally resize all elements inside subgroups*/
GroupHandler.prototype.multiSelectResizeGrpElm=function(parentGroup, group, newRect, rotation){
	var parGrpElmLen = parentGroup.elements.length;
	for (var i = 0; i < parGrpElmLen; i++) {
		var id =  parentGroup.elements[i].id;
		if(TemplateUtils.isElementTypeGroup(id))
		{
			this.multiSelectResizeGrpElm(groupHandler.groupMap[id], group, newRect, rotation);
		}
		else
		{
			var element = parentGroup.elements[i];
			this.resizeNonGrpElm(element, group, newRect, rotation);
		}
	}
};

/*On group resize, proportionally resize all groups and elements*/
GroupHandler.prototype.resizeGrpElm=function(groupObj, newRect, rotation)
{
    for (var i = 0; i < this.selGrpElmList.length; i++)
    {
        var element = this.selGrpElmList[i];
        this.resizeNonGrpElm(element, groupObj, newRect, rotation);
    }
};

GroupHandler.prototype.resizeNonGrpElm = function(element, groupObj, newRect, rotation) {
    var rect = this.getRectForElmOnGroupResize(groupObj, newRect, rotation, element);
    log.debug("rect.x:" + rect.x + " rect.y = " + rect.y + " rect.w = " + rect.w + " rect.h = " + rect.h, "resizeGrpElm");
    templateObj.updateObjXYHWFromObj(element.ref, rect);
    var elementType = element.ref.elementType;
    if (TemplateUtils.isElementTypeMainFrame(element.id))
    {
        templateObj.resetMnFrmElementPos(element.id, element.ref);
        if (TemplateUtils.isShape(elementType)) {
            element.ref.shapeDetails.previewMode = true;
            templateObj.redrawShape(element.ref, element.id);
        }
    }
    else
    {
        templateObj.updateBlkFrameDimention(element.id, element.ref.x, element.ref.y, element.ref.w, element.ref.h);
    }
};

/*iterate recursively and check whether the any of the sub group and its elements is getting outside main frame boundary when the group(grpId) is resized and positioned to new cordinates newRect */
GroupHandler.prototype.multiSelChkElmMnFrmBdryForGrpResize=function(parentGroup, group, newRect, rotation)
{
	var result = true;
	var parGrpElmLen = parentGroup.elements.length;
	for (var i = 0; i < parGrpElmLen; i++) {
		var id =  parentGroup.elements[i].id;
		if(TemplateUtils.isElementTypeGroup(id))
		{
			result = this.multiSelChkElmMnFrmBdryForGrpResize(groupHandler.groupMap[id], group, newRect, rotation);
			if(result == false)
			{
				return result;
			}
		}
		else
		{
			var element = parentGroup.elements[i];
			var rect = this.getRectForElmOnGroupResize(group, newRect, rotation, element);
			result = this.checkBoundaryCondition(rect, element.ref.rotation);
			if(result == false)
			{
				log.debug("ERROR multiSelChkElmMnFrmBdryForGrpResize return false", "multiSelChkElmMnFrmBdryForGrpResize");
				return result;
			}
		}
	}
	
	return result;
};

/*check whether the elements of the selected group is getting outside main frame boundary when the group(grpId) is resized and positioned to new cordinates newRect. The selGrpElmList is populated in touch start event of control point*/
GroupHandler.prototype.checkElmMnFrmBoundaryForGroupResize=function(groupObj, newRect, rotation)
{
	var result = true;
	for (var i = 0; i < this.selGrpElmList.length; i++) 
	{
		var element = this.selGrpElmList[i];
		var rect = this.getRectForElmOnGroupResize(groupObj, newRect, rotation, element);
		result = this.checkBoundaryCondition(rect, element.ref.rotation);
		var curElmObjPoints ={mainFrameSelElmObj:{x:rect.x,y:rect.y,w:rect.w,h:rect.h,rotation:(element.ref.rotation)},mainFrame:templateObj.mainFrame};
		if(result == false)
		{
			log.debug("ERROR checkElmMnFrmBoundaryForGroupResize return false", "checkElmMnFrmBoundaryForGroupResize");
			break;
		}
	}
	
	return result;
};

/*check whether the elements of the selected group is getting outside main frame boundary when the group(grpId) is resized and positioned to new cordinates newRect. The selGrpElmList is populated in touch start event of control point*/
/*GroupHandler.prototype.chkGrpElmResizeForMnFrmBoundary=function()
{
	var result = true;
	for (var i = 0; i < this.selGrpElmList.length; i++) 
	{
		var element = this.selGrpElmList[i];
		var rect =
		{
			x:element.ref.x,
			y:element.ref.y,
			w:element.ref.w,
			h:element.ref.h
		};
		result = this.checkBoundaryCondition(rect, element.ref.rotation);
		if(result == false)
		{
			log.debug("ERROR chkGrpElmResizeForMnFrmBoundary return false", "chkGrpElmResizeForMnFrmBoundary");
			break;
		}
	}
	return result;
};*/

/*On element resize, proportionally resize group having groupId. For calculation bring the group to the center and unrotate the group and all its elements and then calculate outer rect for the group and calculate group center
 , re-rotate the group center with respect to origin and translate back the group center to its original position*/
GroupHandler.prototype.resizeGrpOnElmResize=function(groupId)
{
	var parentGrpId = this.findRoot(groupId);
	var group = this.groupMap[parentGrpId];
	var mainFrmOffset=$("#"+FRAME_ID_PREFIX+"Main").offset();
	var groupCenter = {
			x:group.x + group.w/2 + mainFrmOffset.left,
			y:group.y + group.h/2 + mainFrmOffset.top
	};
	var origin = {
			x:0,
			y:0
	};
	var rect ={minX:Number.MAX_VALUE, 
			minY:Number.MAX_VALUE, 
			maxX:Number.MIN_VALUE, 
			maxY:Number.MIN_VALUE};
	
	this.selGrpElmList=[];
	this.getGroupElements(parentGrpId, this.selGrpElmList);
	for (var i = 0; i < this.selGrpElmList.length; i++) 
	{
		var elmObj = this.selGrpElmList[i].ref;
		var trElmCenter = {
				x:elmObj.x + elmObj.w/2 + mainFrmOffset.left - groupCenter.x,
				y:elmObj.y + elmObj.h/2 + mainFrmOffset.top - groupCenter.y
		};
		var unrotatedElmCenter = this.rotationUtil.doTransformation(origin, trElmCenter,-group.rotation, true);
		var elmRect = {
			x:unrotatedElmCenter.x - elmObj.w/2 ,
			y:unrotatedElmCenter.y - elmObj.h/2,
			w:elmObj.w,
			h:elmObj.h
		};
		if(rect.minX > elmRect.x)
		{
			rect.minX = elmRect.x;
		}
		if(rect.minY > elmRect.y)
		{
			rect.minY = elmRect.y;
		}
	    if(rect.maxX < (elmRect.x + elmRect.w))
	    {
	    	rect.maxX = elmRect.x + elmRect.w;
	    }
	    if(rect.maxY < (elmRect.y + elmRect.h))
	    {
	    	rect.maxY = elmRect.y + elmRect.h;
	    }
	}
	
	var gw= rect.maxX - rect.minX;
	var gh= rect.maxY - rect.minY;
	var updatedGrpCenter = {
			x:rect.minX + gw/2,
			y:rect.minY + gh/2
	};
	var rotatedGrpCenter = this.rotationUtil.doTransformation(origin, updatedGrpCenter,group.rotation, true);
	group.x= rotatedGrpCenter.x - gw/2 + groupCenter.x - mainFrmOffset.left;
	group.y= rotatedGrpCenter.y - gh/2 + groupCenter.y - mainFrmOffset.top;
	group.w = gw;
	group.h = gh;
	this.updateGrpElmObj(0, true);
	log.debug("group.x = " + group.x + " group.y = " + group.y + " group.w = " + group.w + " group.h = " + group.h + " rotation = " + group.rotation + " length = " + this.selGrpElmList.length + " groupId = " + groupId + " parentGrpId = " + parentGrpId,"resizeGrpOnElmResize");
	this.selGrpElmList=[];
};
/* To check whether group can be formed with the multiselected elements considering conditions like all elements must be same for a group*/
GroupHandler.prototype.isGroupCreationValidForSelectedElements=function()
{
	var multiSelArr = templateObj.multiSelElmArr;
	if(multiSelArr.length>1)
	{
		var elementType = groupHandler.getElementTypeForGrping(multiSelArr[0]);
		for(var elmObjInd  in multiSelArr)
		{
			if(groupHandler.getElementTypeForGrping(multiSelArr[elmObjInd]) != elementType)
			{
				return false;
			}
		}
		return true;
	} 
	else
	{
		return false;	
	}
	
};
GroupHandler.prototype.getElementTypeForGrping=function(elementObj)
{
	var elmntId=elementObj.id;
    if(TemplateUtils.isElementTypeGroup(elmntId))
    {
        return groupHandler.getElementTypeForGrping(elementObj.ref.elements[0]);
    }
    else if(TemplateUtils.isElementTypeMainFrame(elmntId))
    {
        var elementType = elementObj.ref.elementType;
        return getBaseElementTypeForSimilarCategoryElmnts(elementType);
    }
    else
    {
        return elementObj.ref.element[0].elementType;
    }
};

function getBaseElementTypeForSimilarCategoryElmnts(elementType)
{
	if(elementType == undefined)
	{
		return TEXT;
	}
	else if(TemplateUtils.isShape(elementType))
	{
		return SHAPE;
	}
	else if(elementType>=widgetInfo.ANALOG_CLOCK_1 && elementType<=widgetInfo.ANALOG_CLOCK_5)
	{
		return widgetInfo.ANALOG_CLOCK_1;
	}
	else if(elementType>=widgetInfo.WEATHER_TODAY && elementType<=widgetInfo.WEATHER_HOUR)
	{
		return widgetInfo.WEATHER_TODAY;
	}
	else
	{
		return elementType;
	}
}

var groupHandler = new GroupHandler();

