/*
 * Created by amber.m 08-11-2016 for element rotation utility functions
 * Converted to js Rotation logic by sachin bisht
 *
 */

var RotationUtil = function(){
    
};

RotationUtil.prototype.getCenter=function(rect)
{
    var center = {
        x:0,
        y:0
    };
    center.x = rect.x + (rect.w) / 2;
    center.y = rect.y + (rect.h) / 2;
    return center;
};
//Why we need this function???
RotationUtil.prototype.getGroupCenter=function(groupRect)
{
    // internal calculation to find the actual center of group (by removing offsets)
    var groupOffset = 1.5; //offset in premium author
    var rect = {
    x:0,
    y:0,
    w:0,
    h:0
    };
    rect.x = groupRect.x + groupOffset;
    rect.y = groupRect.y + groupOffset;
    rect.w = groupRect.w - 2 * (groupOffset);
    rect.h = groupRect.h - 2 * (groupOffset);
    return this.getCenter(rect);
};

RotationUtil.prototype.getTransformedPoint=function(centerOfRotation/*group center*/, originalElementCenter, originalTopLeftCorner, angleInDegree){
    //converting angle from degrees to radian as per the available api
    var angleInRadian = (Math.PI / 180) * angleInDegree;
    
    //rotate top left point about group axis
    var transformedLeftCornerIntermediate = this.doTransformation(centerOfRotation, originalTopLeftCorner, angleInRadian);
    
    //rotate center point of element about group center axis
    var transformedElementCenter = this.doTransformation(centerOfRotation, originalElementCenter, angleInRadian);
    
    //backRotation of top left corner about transformed element center axis
    var transformedLeftCorner = this.doTransformation(transformedElementCenter, transformedLeftCornerIntermediate, -1*angleInRadian);
    
    return transformedLeftCorner;
};

RotationUtil.prototype.doTransformation=function(centerOfRotation, pointToTransform, angleInRadian,isAngleInDeg){
//    log.debug("1angleInRadian: "+ angleInRadian+",isAngleInDeg:"+isAngleInDeg,"doTransformation");
    if(isAngleInDeg!=undefined && isAngleInDeg)
    {
        angleInRadian=angleInRadian*(Math.PI/180);
    }

    if(angleInRadian != 0){
        //Setting Constants
        var r00 = Math.cos(angleInRadian);
        var r01 = -1*Math.sin(angleInRadian);
        var r10 = -1*r01;
        var r11 = r00;
        var point = {
            x:0,
            y:0
        };
        point.x = r00 * pointToTransform.x + r01 * pointToTransform.y + centerOfRotation.x - r00 * centerOfRotation.x - r01 * centerOfRotation.y;
        point.y = r10 * pointToTransform.x + r11 * pointToTransform.y + centerOfRotation.y - r10 * centerOfRotation.x - r11 * centerOfRotation.y;
        return point;
    }else{
        return pointToTransform;
    }
    
};

RotationUtil.prototype.testRotationUtil=function()
{
    //input
    var groupRect = {
    x:-1.5,
    y:-1.5,
    w:1003,
    h:1003
    };

    var rect = {
    x:0,
    y:0,
    w:500,
    h:1000
    };
    
    //Element
    var originalTopLeftCornerPoint = {
    x:0,
    y:0
    };
    
    var angleInDegree = 216;
    
    //Group
    var groupCenterPoint = this.getGroupCenter(groupRect);
    
    
  
    var elementCenterPoint = this.getCenter(rect);
    
    //Transforming element top left corner
    var transformedPoint = this.getTransformedPoint(groupCenterPoint, elementCenterPoint, originalTopLeftCornerPoint, angleInDegree);
    
    log.debug("transformedPoint x = "+transformedPoint.x+", y = "+transformedPoint.y ,"testRotationUtil");
    var element = {
    x:3,
    y:0,
    w:6,
    h:6,
    rotation:20
    };
    var P = {
    x:3,
    y:0
    };
    var val = this.pointInsideRect(element, P);
    //alert("transformedPoint x = "+transformedPoint.x+", y = "+transformedPoint.y +"testRotationUtil Point Inside rect = "+val);
};
/*Element must have x, y, w, h & rotation parameters. This function will check the point is inside the transformed/rotated rect or not.
 * P point to be tested
 */
RotationUtil.prototype.pointInsideRect=function(element, P)
{
    //rotated rectangle coordinates in counter clock wise direction (this is important for the logic as we are checking counter clock wise direction)
    var scale = templateObj.mainFrame.scale;
    var mainFrmOffset=$("#"+FRAME_ID_PREFIX+"Main").offset();
    var scaledElement = {x:(mainFrmOffset.left + element.x*scale),y:(mainFrmOffset.top + element.y*scale),w:(element.w*scale),h:(element.h*scale),rotation:element.rotation};
    return this.pointInsideRectWithOutScale(scaledElement,P);

};
RotationUtil.prototype.pointInsideRectWithOutScale=function(element, P)
{
    //rotated rectangle coordinates in counter clock wise direction (this is important for the logic as we are checking counter clock wise direction)
    var angleInRadian = (Math.PI / 180) * element.rotation;
    var center = this.getCenter(element);
    var A = {x:element.x, y:element.y};
    var B = {x:element.x+element.w , y:element.y};
    var C = {x:element.x+element.w , y:element.y+element.h};
    var D = {x:element.x, y:element.y+element.h};
    A = this.doTransformation(center, A, angleInRadian);
    B = this.doTransformation(center, B, angleInRadian);
    C = this.doTransformation(center, C, angleInRadian);
    D = this.doTransformation(center, D, angleInRadian);
    var isPointInside = false;

    //need to check for all the sides (can be used for any polygon)
    if (this.isOrientationCounterClockWise(P, A, B) && this.isOrientationCounterClockWise(P, B, C) && this.isOrientationCounterClockWise(P, C, D) && this.isOrientationCounterClockWise(P, D, A))
    {
        isPointInside = true;
    }
    return isPointInside;
};

/*Checking orietation of point, checking the orientation of p1 around the line p2~p3.*/
RotationUtil.prototype.isOrientationCounterClockWise=function(p1, p2, p3)
{
    // > 0 (Point lies anti clock wise),
    // = 0 (Point lies within the line),
    // < 0 (Point lies clock wise)
    return ((p3.y - p2.y) * (p2.x - p1.x) - (p2.y - p1.y) * (p3.x - p2.x)) >= 0;
};
//alert("Rotation Util");

// Getting ClockwiseTransformed Rotated Point array from an element
RotationUtil.prototype.getClkwisTranforRotatedPointsArrOfElmnt=function(elmnt)
{
    var center = this.getCenter(elmnt);
    var xoffsetArr = [0,elmnt.w,elmnt.w,0];
    var yoffsetArr = [0,0,elmnt.h,elmnt.h];
    var itr,pointArr=[];
    var angleInRadian =(Math.PI / 180) * (elmnt.rotation == undefined?0:elmnt.rotation);
    for(itr=0;itr<4;itr++)
    {
    	var samplePoint = {x:(elmnt.x+xoffsetArr[itr]),y:(elmnt.y+yoffsetArr[itr])};
    	pointArr.push(this.doTransformation(center, samplePoint,angleInRadian));
    }
    return pointArr;
};
var utilR = new RotationUtil();
//utilR.testRotationUtil();
