/* 
 * Created by amber.m 05-08-2015
 * Created to find empty area rect in current canvas for RMD Template
 *  Note :: Order of function call should be 
 *  1. loadTemplateAreaObj : - fill the x and y point map
 *  2. loadEndPointArray :- create x and y end point sorted array(all distinct points)
 *  3. loadAreaMatrix :- create and load area matrix(using x and y end point array) & template objects, occupied area represented by 1 & empty area by 0
 *  4. findEmptyRects :- depending upon the touch point returns the array of distict EmptyArea objects
 *
 */


var TemplateArea=function(){
    this.xPointMap={};
    this.yPointMap={};
    this.xPointArr=[];
    this.yPointArr=[];
    this.areaMatrix = null;
    this.emptyRectArr=[];
    this.touchX=-1;
    this.touchY=-1;
    this.touchPtR=-1;
    this.touchPtC=-1;
};


TemplateArea.prototype.setInXMap=function(val){
    if(this.xPointMap[val] == undefined){
        this.xPointMap[val] = 1;
    }else{
        this.xPointMap[val]++;
    }
};

TemplateArea.prototype.setInYMap=function(val){
    if(this.yPointMap[val] == undefined){
        this.yPointMap[val] = 1;
    }else{
        this.yPointMap[val]++;
    }
};

TemplateArea.prototype.delFromXMap=function(val){
    if(this.xPointMap[val] != undefined){
        if(this.xPointMap[val]>1){
            this.xPointMap[val]--;
        }else{
            delete this.xPointMap[val];
        }
    }
};

TemplateArea.prototype.delFromYMap=function(val){
    if(this.yPointMap[val] != undefined){
        if(this.yPointMap[val]>1){
            this.yPointMap[val]--;
        }else{
            delete this.yPointMap[val];
        }
    }
};

//creating sorted array of frame/element end points
TemplateArea.prototype.reloadXYPointArr=function(){
    if(this.xPointArr.length > 0 || this.yPointArr.length > 0){
        this.xPointArr=[];
        this.yPointArr=[];
    }
    //putting unique x and y points in to point array
    for(var key in this.xPointMap){
        this.xPointArr.push(parseFloat(key));
    }
    for(var key in this.yPointMap){
        this.yPointArr.push(parseFloat(key));
    }
    this.xPointArr.sort(function(a,b){return a-b;});
    this.yPointArr.sort(function(a,b){return a-b;});
};

TemplateArea.prototype.findIndexOfNumberInArr=function(arrType, number) {
    var min = 0;
    var currIndex;
    var currEle;
    var arr = null;
    if(arrType == "X"){
        arr = this.xPointArr;
    }else{
        arr = this.yPointArr;
    }
    var max = arr.length - 1;
    
    while (min <= max) {
        currIndex = (min + max) / 2 | 0;
        currEle = arr[currIndex];
        if (currEle < number) {
            min = currIndex + 1;
        }
        else if (currEle > number) {
            max = currIndex - 1;
        }
        else {
            return currIndex;
        }
    }
    
    return -1;
};

TemplateArea.prototype.resetAreaMatrix=function(){
    var col = this.xPointArr.length;
    var row = this.yPointArr.length;
    if(this.areaMatrix != null){
        this.areaMatrix = null;
    }
    this.areaMatrix = new Array(row-1);
    for(var i=0;i<row-1;i++){
        this.areaMatrix[i] = new Array(col-1);
        for(var j=0;j<col-1;j++){
            this.areaMatrix[i][j] = 0;
        }
    }
};

TemplateArea.prototype.setAreaMatrixForObj=function(obj){
    var stX = -1,endX = -1,stY = -1,endY = -1;
    stX = this.findIndexOfNumberInArr("X",obj.x);
    endX = this.findIndexOfNumberInArr("X",obj.x+obj.w);
    stY = this.findIndexOfNumberInArr("Y",obj.y);
    endY = this.findIndexOfNumberInArr("Y",obj.y+obj.h);
    log.debug("MATRIX POINT INPUT = "+stX+" "+stY+" "+endX+" "+endY, "setAreaMatrix");
    if(this.areaMatrix != null){
        if(stX != -1 && stY != -1 && endX != -1 && endY != -1){
            for(var i=stY;i<endY;i++){
                for(var j=stX;j<endX;j++){
                    this.areaMatrix[i][j] = 1;
                }
            }
        }else{
            log.debug("INCORRECT MATRIX POINT INPUT = "+stX+" "+stY+" "+endX+" "+endY, "setAreaMatrix");
        }
    }
};

TemplateArea.prototype.getSetTouchPtRowCol=function(){
    if(this.touchX != -1 && this.touchY != -1){
        var col = this.xPointArr.length-1;
        var row = this.yPointArr.length-1;
        for(var r = 0;r<row;r++){
            for(var c = 0;c<col;c++){
                if(this.touchX >= this.xPointArr[c] && this.touchX < this.xPointArr[c+1] && this.touchY >= this.yPointArr[r] && this.touchY < this.yPointArr[r+1]){
                    this.touchPtR = r;
                    this.touchPtC = c;
                    break;
                }
            }
        }
    }
    log.debug(" this.touchPtR = "+this.touchPtR+" this.touchPtC="+this.touchPtC, "getSetTouchPtRowCol");
};

//get the last empty col index for start input r and c, covering the x touch point, else return -1
TemplateArea.prototype.getEndX=function(r, c){
    var mat = this.areaMatrix;
    var col = this.xPointArr.length-1;
    var i = c;
    for(i = c;i<col;i++){
        if(mat[r][i] != 1){
            continue;
        }else{
            break;
        }
    }
    if(this.xPointArr[i]>this.touchX){
        return i-1;
    }
    return -1;
};

//get the last empty row index for start input r and c, covering the y touch point, else return -1
TemplateArea.prototype.getEndY=function(r, c){
    var mat = this.areaMatrix;
    var row = this.yPointArr.length-1;
    var i = r;
    for(i = r;i<row;i++){
        //log.debug("r"+i+" c ="+c +" mat[i][c] = "+mat[i][c],"getEndY");
        if(mat[i][c] != 1){
            continue;
        }else{
            break;
        }
    }
    if(this.yPointArr[i]>this.touchY){
        return i-1;
    }
    return -1;
};

TemplateArea.prototype.addToEmptyRectArray=function(x, y, w, h){
    var lt = {x:x,y:y};
    var rb = {x:x+w,y:y+h};
    var len = this.emptyRectArr.length;
    var bDuplicate = false;
    for(var i=0;i<len;i++){
        if(isPointInsideRect(lt, this.emptyRectArr[i]) && isPointInsideRect(rb, this.emptyRectArr[i]))
        {
            bDuplicate = true;
            break;
        }
    }
    if(!bDuplicate){
        var rect = new EmptyRect(x, y, w, h);
        this.emptyRectArr.push(rect);
    }
};


TemplateArea.prototype.toString=function(){
    log.debug(JSON.stringify(this),"TEMPLATE AREA OBJECT");
};

/*----EMPTY RECT OBJ-----*/
var EmptyRect=function(x, y, w, h){
    this.x=x;
    this.y=y;
    this.w=w;
    this.h=h;
};

/*----TEMPLATE INTERFACE FUNCTION-----*/
template.prototype.loadTemplateAreaObj=function(){
    log.debug("CALLED", "loadTemplateAreaObj");
    this.resetTemplateAreaObj();
    var frameId = null;
    var frame = null;
    var tAreaObj = this.templateAreaRMD;
    
    log.debug(" STARTED ", "loadTemplateAreaObj");
    //push main frame pts
    
    tAreaObj.setInXMap(0);
    tAreaObj.setInXMap(this.mainFrame.w);
    tAreaObj.setInYMap(0);
    tAreaObj.setInYMap(this.mainFrame.h);
    
    //Put all inner frame points into arr
    var innerFrameZorderArray = this.getInnerFrameGroupZorderArray();
    for(var index=0;index<=innerFrameZorderArray.length-1;index++){
        frameId = FRAME_ID_PREFIX+innerFrameZorderArray[index];
        frame = this.getInnerFrameForId(frameId);
        this.addEndPtsForObj(frame);
    }
    
    //Put all floating element points into arr
    var elmCount=this.mainFrame.element.length;
	var elmnt;
	for(var i=0;i<elmCount;i++)
	{
		elmnt=this.mainFrame.element[i];
        if(elmnt != null){
            if(this.isTxtTypeElm(elmnt.elementType)){
                elmnt = elmnt.textElms[0];
            }
            //log.debug(elmnt.x+" "+elmnt.y+" "+elmnt.w+" "+elmnt.h ,"loadTemplateAreaObj");
            this.addEndPtsForObj(elmnt);
        }
    }
    log.debug(" COMPLETED ", "loadTemplateAreaObj");

    //this.loadEndPointArray();
    //this.loadAreaMatrix();
};

template.prototype.resetTemplateAreaObj=function(){
    this.templateAreaRMD = new TemplateArea();
};

template.prototype.removeEndPtsForObj=function(obj){
    var tAreaObj = this.templateAreaRMD;
    if(obj != null){
        log.debug("OBJ REMOVED = "+obj.x+" "+obj.y+" "+obj.w+" "+obj.h,"removeEndPtsForObj");
        tAreaObj.delFromXMap(obj.x);
        tAreaObj.delFromXMap(obj.x+obj.w);
        tAreaObj.delFromYMap(obj.y);
        tAreaObj.delFromYMap(obj.y+obj.h);
    }
};

template.prototype.addEndPtsForObj=function(obj){
    var tAreaObj = this.templateAreaRMD;
    if(obj != null){
        log.debug("OBJ ADDED = "+obj.x+" "+obj.y+" "+obj.w+" "+obj.h,"addEndPtsForObj");
        tAreaObj.setInXMap(obj.x);
        tAreaObj.setInXMap(obj.x+obj.w);
        tAreaObj.setInYMap(obj.y);
        tAreaObj.setInYMap(obj.y+obj.h);
    }
};

template.prototype.loadEndPointArray=function(){
    var tAreaObj = this.templateAreaRMD;
    if(tAreaObj != null){
        tAreaObj.reloadXYPointArr();
        tAreaObj.toString();
    }
};

template.prototype.loadAreaMatrix=function(){
    var tAreaObj = this.templateAreaRMD;
    tAreaObj.resetAreaMatrix();
    
    var frameId = null;
    var frame = null;
    var innerFrameZorderArray = this.getInnerFrameGroupZorderArray();
    for(var index=0;index<=innerFrameZorderArray.length-1;index++){
        frameId = FRAME_ID_PREFIX+innerFrameZorderArray[index];
        frame = this.getInnerFrameForId(frameId);
        if(this.modelType==RMD_MODEL)
        {
            if(frame.element.length>0 || frame.table != null){
                tAreaObj.setAreaMatrixForObj(frame);
            }
        }else{
            var frameType=this.getFrameType(frameId);
            //if(this.isVideoTypeElm(frameType))
            if(this.checkVideoTypeAsPerModel(frameType))	
            {
                tAreaObj.setAreaMatrixForObj(frame);
            }
        }
    }
    
    if(this.modelType==RMD_MODEL)
    {
        var elmCount=this.mainFrame.element.length;
        var elmnt;
        for(var i=0;i<elmCount;i++)
        {
            elmnt=this.mainFrame.element[i];
            if(elmnt != null){
                if(this.isTxtTypeElm(elmnt.elementType)){
                    elmnt = elmnt.textElms[0];
                }
                //log.debug(elmnt.x+" "+elmnt.y+" "+elmnt.w+" "+elmnt.h ,"loadTemplateAreaObj");
                tAreaObj.setAreaMatrixForObj(elmnt);
            }
        }
    }
    tAreaObj.toString();
    //this.findEmptyRects(40, 300);
};

//Input touchX and touchY must be relative to mainFrame
//X point array representing column points and y point array representing row points
template.prototype.findEmptyRects=function(touchX, touchY){
    var tAreaObj = this.templateAreaRMD;
    tAreaObj.touchX = touchX;
    tAreaObj.touchY = touchY;
    tAreaObj.getSetTouchPtRowCol();
    
    if(tAreaObj.emptyRectArr.length >0){
        tAreaObj.emptyRectArr = [];
    }
    
    if(tAreaObj.areaMatrix[tAreaObj.touchPtR][tAreaObj.touchPtC]==1){
        log.debug("TOUCH POINT OVER OBJECT","findEmptyRects");
        return tAreaObj.emptyRectArr;
    }
    var mat = tAreaObj.areaMatrix;
    var col = tAreaObj.xPointArr.length-1;
    var row = tAreaObj.yPointArr.length-1;
    
    for(var r = 0;r<row;r++){
        for(var c = 0;c<col;c++){
            //log.debug("r="+r+" c="+c +" mat[r][c]="+mat[r][c]+" tAreaObj.xPointArr[c] = "+tAreaObj.xPointArr[c]+" tAreaObj.yPointArr[r]="+tAreaObj.yPointArr[r],"findEmptyRects");
            if(mat[r][c] == 1 || tAreaObj.xPointArr[c]>tAreaObj.touchX || tAreaObj.yPointArr[r]>tAreaObj.touchY){
                continue;
            }
            //log.debug("r="+r+" c="+c,"findEmptyRects");
            this.subFindEmptyRects(r, c);
        }
    }
    
    tAreaObj.toString();
    return tAreaObj.emptyRectArr;
};


template.prototype.subFindEmptyRects=function(r, c){
    var tAreaObj = this.templateAreaRMD;
    var mat = tAreaObj.areaMatrix;
    var etX = tAreaObj.getEndX(r,c);
    var etY = tAreaObj.getEndY(r,c);
    log.debug("etX="+etX+" etY="+etY,"subFindEmptyRects");
    
    if(etX != -1 && etY != -1){
        for(var row = r;row<=etY;row++){
            for(var col = c;col<=etX;col++){
                if(mat[row][col] == 0 && col == etX && row == etY){
                    //save rect
                    if(col >=tAreaObj.touchPtC){
                        tAreaObj.addToEmptyRectArray(tAreaObj.xPointArr[c], tAreaObj.yPointArr[r], tAreaObj.xPointArr[etX+1]-tAreaObj.xPointArr[c], tAreaObj.yPointArr[etY+1]-tAreaObj.yPointArr[r]);
                        
                        log.debug("CASE 1 r= "+r+" c = "+c+" row="+row+" col="+col, "subFindEmptyRects");
                    }
                }else if(mat[row][col] == 0){
                    continue;
                }else if(mat[row][col] == 1 && row<=tAreaObj.touchPtR && col<= tAreaObj.touchPtC){
                    log.debug("NO Possible rect at r = "+r+"  c= "+c,"subFindEmptyRects");
                    return;
                }else if(mat[row][col] == 1 && (row-1)>=tAreaObj.touchPtR && etX >=tAreaObj.touchPtC){
                    //save prev rect
                    tAreaObj.addToEmptyRectArray(tAreaObj.xPointArr[c], tAreaObj.yPointArr[r], tAreaObj.xPointArr[etX+1]-tAreaObj.xPointArr[c], tAreaObj.yPointArr[row]-tAreaObj.yPointArr[r]);
                    log.debug("CASE 2 r= "+r+" c = "+c+" row="+(row-1)+" col="+etX, "subFindEmptyRects");
                    etX = col-1;
                    row = r;
                }else{
                    log.debug("Reduces end col","subFindEmptyRects");
                    etX = col-1;
                    row = r;
                }
                
            }
        }
    }
    
};

template.prototype.getBestEmptyRectRMD=function(){
    this.loadTemplateAreaObj();
    this.loadEndPointArray();
    var tAreaObj = this.templateAreaRMD;
    this.loadAreaMatrix();
    var row = tAreaObj.areaMatrix.length;
    var col = tAreaObj.areaMatrix[0].length;
    var largestRect = {x:0,y:0,w:0,h:0};
    for(var i=0;i<row;i++){
        for(var j=0;j<col;j++){
            //log.debug("i="+i+" j="+j,"getBestEmptyRectRMD");
            if(tAreaObj.areaMatrix[i][j] == 0){
                largestRect = this.findLargestEmptyRectInMatrixPos(tAreaObj.areaMatrix, i, j, largestRect);
            }
        }
    }
    log.debug("largestRect="+largestRect.x+" "+largestRect.y+" "+largestRect.w+" "+largestRect.h,"getBestEmptyRectRMD");
    return largestRect;

};




template.prototype.findLargestEmptyRectInMatrixPos=function(mat, i, j, rect){
    
    var pt = {x:j,y:i};
    var pt2 = {x:j,y:i};
    var bCol = false;
    var bRow = false;
    //log.debug("","findLargestEmptyRectInMatrixPos");
    for(var row = i;row<mat.length;row++){
        if(!bCol){
            for(var col = j;col<mat[0].length;col++){
                //log.debug("mat[row][col] = "+mat[row][col],"findLargestEmptyRectInMatrixPos");
                if(mat[row][col] == 0){
                    pt.x = col;
                }else{
                    bCol = true;
                    break;
                }
                
            }
            bCol = true;
        }
        for(var col = j;col<=pt.x;col++){
            if(mat[row][col] == 0){
                if(col == pt.x){
                    pt.y = row;
                }
            }else{
                bRow = true;
                break;
            }
        }
        if(bRow){
            break;
        }
    }
    bCol = false;
    bRow = false;
    for(var col = j;col<mat[0].length;col++)
    {
        
        if(!bCol){
            for(var row = i;row<mat.length;row++){
                if(mat[row][col] == 0){
                    pt2.y = row;
                }else{
                    bCol = true;
                    break;
                }
            }
            bCol = true;
        }
        for(var row = i;row<=pt2.y;row++){
            if(mat[row][col] == 0){
                if(row == pt2.y){
                    pt2.x = col;
                }
            }else{
                bRow = true;
                break;
            }
        }
        if(bRow){
            break;
        }
    }
    //log.debug("j"+j+" i "+i+" pt.x = "+pt.x+" pt.y="+pt.y,"findLargestEmptyRectInMatrixPos");
    //log.debug("j"+j+" i "+i+" pt2.x = "+pt2.x+" pt2.y="+pt2.y,"findLargestEmptyRectInMatrixPos");
    var area1 = this.getAreaOfEmptyRect(i, j, pt);
    var area2 = this.getAreaOfEmptyRect(i, j, pt2);
    var currArea = rect.w*rect.h;
    //log.debug("area1="+area1+" area2 ="+area2,"findLargestEmptyRectInMatrixPos");
    var tAreaObj = this.templateAreaRMD;
    if(area1>area2){
        if(area1>currArea){
            rect.x = tAreaObj.xPointArr[j];
            rect.w = tAreaObj.xPointArr[pt.x+1] - rect.x;
            rect.y= tAreaObj.yPointArr[i];
            rect.h = tAreaObj.yPointArr[pt.y+1] - rect.y;
        }
    }else{
        if(area2>currArea){
            rect.x = tAreaObj.xPointArr[j];
            rect.w = tAreaObj.xPointArr[pt2.x+1] - rect.x;
            rect.y= tAreaObj.yPointArr[i];
            rect.h = tAreaObj.yPointArr[pt2.y+1] - rect.y;
        }
    }
    return rect;
};

template.prototype.getAreaOfEmptyRect=function(i, j, pt){
    var tAreaObj = this.templateAreaRMD;
    var x1 = tAreaObj.xPointArr[j];
    var x2 = tAreaObj.xPointArr[pt.x+1];
    var y1 = tAreaObj.yPointArr[i];
    var y2 = tAreaObj.yPointArr[pt.y+1];
    return (x2 - x1) * (y2 - y1);
};
