define('RectangularCallout', ['Shape'], function (Shape) {/* define module names 'RectangularCallout' with the dependency shape[AMD]*/
'use strict';/* enables code execution in strict mode which shows script errors(bad syntax to real errors)*/
     var prop ={};
     return {
         draw : function (options){/* initializes canvas and draw shape*/
                prop = $.extend(true, {},Shape.default, options); /* adds default properties of shapes*/     
                var d = new $.Deferred();/*creating new deferred object to return promise*/
                Shape.initCanvas(prop.w, prop.h, prop.shapeDetails.FillColor);/* initializes canvas*/
                var context = Shape.context;
                this.drawOutline(prop,true,'');
                 var $this = this;   
                
                 Shape.fillImage(prop).done(function(value) {/*fills background image for shape*/
                      if(prop.shapeDetails.FillRange==1){
                        $this.drawOutline(prop,true,'source-over');/*for drawing border separately over shape to get rid of opacity issue with border*/
                      }else{
                          $this.drawOutline(prop,true,'');
                      }
                    context.save();
                    d.resolve(Shape.canvas.toDataURL('data:image/jpg;'));/*returns an object of shape drawn on canvas*/
                });   
                 
                return d.promise();
         },
         drawOutline : function (prop, borderFlag, composite){/*draws shape*/
                var context = Shape.context;
                context.globalAlpha = 1;
                if(composite){
                    context.globalCompositeOperation = composite; 
                 }
                var borderWidth = Shape.getFigureStrokeWidth(prop.shapeDetails.FigureStrokeWidth);

                if (prop.shapeDetails.FigureRoundRatio >= 100) {
                    this.drawEllipse(context, prop, composite);
                    return;
                }

                var width = Shape.width;
                var height = Shape.height;

                /*if (prop.shapeDetails.FigureRoundRatio <= ((7.5 / (width / 2)) * 100)) {
                    prop.shapeDetails.FigureRoundRatio = 0;
                }*/

                var w = (prop.shapeDetails.InnerRect.w / 100) * Shape.width;
                var h = (prop.shapeDetails.InnerRect.h / 100) * Shape.height;
                //radius = (h > w) ? (w / 2) : (h / 2);
                var radius = Math.min(10, prop.shapeDetails.FigureRoundRatio);
                //var r = radius * roundRatio / 100;
                var x = Shape.canvasMargin/2 + (prop.shapeDetails.InnerRect.x / 100) * width;
                var y = Shape.canvasMargin/2 + (prop.shapeDetails.InnerRect.y / 100) * height;

                var r = x + w;
                var b = y + h;

                var px = (prop.shapeDetails.CalloutPoint.x / 100) * width;
                var py = (prop.shapeDetails.CalloutPoint.y / 100) * height;

                var calloutDir = getParseIntInDec(prop.shapeDetails.CalloutPoint.direction); // convert to integer
                var ratioOffset = (radius / 10);
                if (ratioOffset == 0) {
                    ratioOffset = 1;
                }
                //var ratioOffsetR = ratioOffsetL;
                var con1, con2;
                var dir;

                switch (calloutDir) {

                    case 0:
                        con1 = y + 0.2 * h * ratioOffset;
                        con2 = y + 0.5 * h - ratioOffset * 0.1 * h;
                        dir = 0;
                    break;
                    case 1:
                        con1 = x + 0.2 * w * ratioOffset;
                        con2 = x + 0.5 * w - ratioOffset * 0.1 * w;
                        dir = 2;
                    break;
                    case 2:
                        con1 = x + 0.5 * w + ratioOffset * 0.1 * w;
                        con2 = x + 1 * w - ratioOffset * 0.2 * w;
                        dir = 2;
                    break;
                    case 3:
                        con1 = y + 0.2 * h * ratioOffset;
                        con2 = y + 0.5 * h - ratioOffset * 0.1 * h;
                        dir = 1;
                    break;
                    case 4:
                        con1 = y + 0.5 * h + ratioOffset * 0.1 * h;
                        con2 = y + 1 * h - ratioOffset * 0.2 * h;
                        dir = 1;
                    break;
                    case 5:
                        con1 = x + 0.5 * w + ratioOffset * 0.1 * w;
                        con2 = x + 1 * w - ratioOffset * 0.2 * w;
                        dir = 3;
                    break;
                    case 6:
                        con1 = x + 0.2 * w * ratioOffset;
                        con2 = x + 0.5 * w - ratioOffset * 0.1 * w;
                        dir = 3;
                    break;
                    case 7:
                        con1 = y + 0.5 * h + ratioOffset * 0.1 * h;
                        con2 = y + 1 * h - ratioOffset * 0.2 * h;
                        dir = 0;
                    break;
                    default:
                    break;

                }

                context.beginPath();
                context.moveTo(x+radius,y);

                if (dir == 2) {
                    context.lineTo(con1,y);
                    context.lineTo(px,py);
                    context.lineTo(con2,y);
                    context.lineTo(r-radius,y);
                }
                else {
                    context.lineTo(r-radius,y);
                }
                context.quadraticCurveTo(r,y,r,y+radius);

                if (dir == 1) {
                    context.lineTo(r,con1);
                    context.lineTo(px,py);
                    context.lineTo(r,con2);
                    context.lineTo(r,b-radius);
                }
                else {
                    context.lineTo(r,b-radius);
                }
                context.quadraticCurveTo(r, b, r-radius, b);

                if (dir == 3) {
                    context.lineTo(con2,b);
                    context.lineTo(px,py);
                    context.lineTo(con1,b);
                    context.lineTo(x+radius,b);
                }
                else {
                    context.lineTo(x+radius,b);
                }
                context.quadraticCurveTo(x, b, x, b-radius);

                if (dir == 0) {
                    context.lineTo(x,con2);
                    context.lineTo(px,py);
                    context.lineTo(x,con1);
                    context.lineTo(x,y+radius);
                }
                else {
                    context.lineTo(x,y+radius);
                }
                context.quadraticCurveTo(x, y, x+radius, y);

                Shape.drawStroke(prop);   //draws above lines/paths on canvas
         },



         drawEllipse: function(context, prop, composite) {
              var w = (prop.shapeDetails.InnerRect.w / 100) * Shape.width,
              h = (prop.shapeDetails.InnerRect.h / 100) * Shape.height,
              context = Shape.context,
              radius = w/2,
              radiusH = h/2,
              centreX = radius + (prop.shapeDetails.InnerRect.x / 100) * Shape.width,
              centreY = radiusH + (prop.shapeDetails.InnerRect.y / 100) * Shape.height,
              yellwPerstart = 0, yellwPerEnd = 360,/*defaults*/
              canvasMargin = Shape.canvasMargin/2,
              counterClockwise = false;
              Shape.resetAlpha(composite);
              var activeFrame = templateObj.getActiveFrame();
              var elementID = templateObj.mainFrame.id+"-elm_"+activeFrame.activeEleIndex+"_0";
              if (prop.shapeDetails.divID != null || prop.shapeDetails.divID != undefined) {
                  elementID = prop.shapeDetails.divID;
              }
              var px = (prop.shapeDetails.CalloutPoint.x / 100) * Shape.width;
              var py = (prop.shapeDetails.CalloutPoint.y / 100) * Shape.height;
              var centre = templateObj.getCentreOfEllipse(elementID);
              var m = centre.x + px;
              var n = centre.y + py;
              var ellipsePoint = templateObj.getPointOnEllipse(prop, elementID, m, n, true);
              var angleOffset = templateObj.degreesToRadians(Math.max(ellipsePoint.angleOffset, 8));
              var radians = templateObj.degreesToRadians(ellipsePoint.angle);
              var startAngle = radians  + angleOffset;
              var endAngle = radians - angleOffset;
              context.beginPath();
              context.ellipse(centreX+canvasMargin, centreY+canvasMargin, radius, radiusH, 2*Math.PI, startAngle, endAngle, counterClockwise);
              context.lineTo(px, py);
              context.closePath();
              Shape.drawStroke(prop);/* draws above lines/paths on canvas*/
         }
     };
});