var SHAPE_CANVAS_MARGIN = 3;
define('Shape', [], function() {

	'use strict';
	var context = null, shapeCacheObj = {};
	return {
		width : 0,
		height : 0,
		context : null,
		canvas : null,
		canvasMargin : SHAPE_CANVAS_MARGIN,
		newWidth:0,
		newHeight:0,
		shapeWithNoBg : [shapeInfo.LINE, shapeInfo.ARC], 
		default : {
			w : 600,
			h : 600,
			shapeDetails : {
				typeName : "Figure Rect",
				FigureRoundRatio : "2",
				LineColor : "rgba(56,93,138,1)",
				FigureStrokeWidth : 10,
				FigureStrokePattern : 0,
				FillColor : "rgba(79,129,189,1)",
				Slope : 50,
				FrameWidth : 30,
				ApexRatio : 50,
			}
		},
		createCanvas : function() {
			if (!context) {
				this.canvas = document.getElementById('canvas') || document.createElement('canvas');
				context = this.canvas.getContext('2d');
				return context;
			}
		},
		initCanvas : function(width, height, fillColor, points) {
                context =  context ||  this.createCanvas();
                this.width = width;
                this.height = height;
                this.newWidth = width+this.canvasMargin;
                this.newHeight = height+this.canvasMargin;
                
                this.context = context;
                this.canvas.width = width+this.canvasMargin;
                this.canvas.height = height+this.canvasMargin;  
                this.fillColor = fillColor||this.fillColor;
				this.ylowPts = points;
		},
		fillImage : function(shapePropObj, nofill) {
			var canvas = this;
			var $this = this;
			if(shapePropObj.shapeDetails.FillRange == 0){
				context.globalAlpha = shapePropObj.shapeDetails.FillOpacity/100;
			}
			context.fillStyle=shapePropObj.shapeDetails.FillColor;
			if(shapePropObj.shapeDetails.previewMode)
            {
                context.fillStyle="rgba(0,0,0,0.3)";
                if(shapePropObj.shapeDetails.FillRange==1)
                {
                    context.fillStyle="rgb(0,0,0)";
                }
            }
 			if (!nofill) {
				context.fill();
			}
			if ((shapePropObj.shapeDetails.FillRange == 1 && !nofill) || templateObj.isCroppedMode) {
				context.globalCompositeOperation = "source-out";	
			} else {
                if(shapePropObj.thumbPath!="" ){
                    context.clip();    
                }
			}
          	var borderWidth = this.getFigureStrokeWidth(shapePropObj.shapeDetails.FigureStrokeWidth)/2;
			context.beginPath();
			this.createOuterRect(shapePropObj, nofill);
                
			var d = $.Deferred();
			if ( typeof shapePropObj == 'object' && !shapePropObj.shapeDetails.previewMode) {
			    if(shapePropObj.thumbPath){
			        //Image exists. Draw shape with background image
			        var thumbImg = document.createElement('img');
                        thumbImg.src = shapePropObj.thumbPath;
                        var t1 = new Date().getTime();
                        thumbImg.onload = function() {
                            context.globalAlpha = shapePropObj.shapeDetails.FillOpacity/100;
                            $this.drawScaledImg(thumbImg, context, shapePropObj);
                            var t2 = new Date().getTime();
                            thumbImg = null;
                            //context.restore();
                            console.log("Time taken"+(t2-t1)/1000);
                            d.resolve('Success');
                        };
                        thumbImg.onerror = function() {
                            d.resolve('Fail');
                        };
			    }else{
			        //No image. Draw basic shape without image
			        d.resolve('Success');
			    }


			} else {
				d.resolve('No image');
			}
			return d.promise();
		},
		drawStroke : function(shapePropObj, isComposite) {
			//if(!this.isPreviewModeValid(shapePropObj.elementType) || !shapePropObj.shapeDetails.previewMode){
				context.lineWidth=this.getFigureStrokeWidth(shapePropObj.shapeDetails.FigureStrokeWidth);
				var strokeAlpha = shapePropObj.shapeDetails.FigureStrokeOpacity/100;
				context.strokeStyle = this.shapeRgbToRgba(shapePropObj.shapeDetails.LineColor, strokeAlpha);
				this.setFigureStrokePattern(shapePropObj.shapeDetails);
				context.stroke();
			//}
		},
		isPreviewModeValid : function(shapeType){
			if(shapeType==shapeInfo.LINE || shapeType==shapeInfo.ARC){
				return false;
			}else{
				return true;
			}
		},
		createRect : function() {
			context.rect(0, 0, this.width, this.height);
			context.fillRect(0, 0, this.width, this.height);
		},
		getPercentageValue : function(PercentageVal, MaxValue) {
			var CalValue = MaxValue * (PercentageVal / 100);
			return CalValue;
		},
	    getFigureStrokeWidth: function (value) {
	        /*
	        *   Note: Actual value of the border width will look bulky in the tablet while drawing.
	        *   So to match with the LFD we need to send the half value of the border width when showing in the tablet template page.
	        */
            return value/2;
        },
		setFigureStrokePattern : function(shapeDetailsObj) {
			var strokeWidth = this.getFigureStrokeWidth(shapeDetailsObj.FigureStrokeWidth);
			switch(shapeDetailsObj.FigureStrokePattern) {
			case 0:
				context.setLineDash([0, 0]);
				break;
			case 1:
				context.setLineDash([3*strokeWidth, strokeWidth]);
				break;
			case 2:
				context.setLineDash([strokeWidth, strokeWidth]);
				break;
			case 3:
				context.setLineDash([3*strokeWidth, strokeWidth, strokeWidth, strokeWidth]);
				break;
			case 4:
				context.setLineDash([3*strokeWidth, strokeWidth, strokeWidth, strokeWidth, strokeWidth, strokeWidth]);
				break;
			default :
				context.setLineDash([0, 0]);
				break;

			}
		},
		resetAlpha : function(composite) {
			this.context.globalAlpha = 1;
            if(composite){
                this.context.globalCompositeOperation = "source-over"; 
             }
		},
		clearCanvas : function() {
			context.clearRect(0, 0, this.width, this.height);
		},
		createOuterRect : function(shapePropObj, nofill){
			//templateObj;
			var $this = this;
            var shapeDetails=shapePropObj.shapeDetails;
            var borderWidth = this.getFigureStrokeWidth(shapeDetails.FigureStrokeWidth)/2;
            var bgColor=shapeDetails.FillColor;
            var alpha=shapeDetails.FillOpacity/100;
            var bgColorAlpha = this.shapeRgbToRgba(bgColor, alpha);
            if(templateObj.isCroppedMode){
                borderWidth=0;
                bgColorAlpha='rgba(0,0,0,0.5)';
                alpha=0.5;

            }
            if ((shapeDetails.FillRange == 1 && shapePropObj.thumbPath == "" && !nofill) || templateObj.isCroppedMode || (shapeDetails.previewMode && shapeDetails.FillRange == 1)) {
                context.rect($this.canvasMargin/2, $this.canvasMargin/2, $this.width, $this.height);
                context.fillStyle =shapeDetails.previewMode?"rgba(0,0,0,0.3)":bgColorAlpha;
			    context.globalAlpha = alpha;
			    context.fill();
			}
			
		},
		drawScaledImg : function(img, ctx, shapeElemObj){
			var cropXYWH= templateObj.getShapesImageCropXYWH(shapeElemObj);
			var w = ctx.canvas.width;
			var h = ctx.canvas.height;
			ctx.drawImage(img, cropXYWH.x, cropXYWH.y, cropXYWH.w, cropXYWH.h,0, 0, w, h);

		},

		shapeRgbToRgba: function(rgbval, alpha){
            var alphaVal = alpha == undefined ? 1 : alpha;
            var new_col;
            if(rgbval.indexOf('rgba')==-1){
                var new_col = rgbval.replace(/rgb/i, "rgba");
                new_col = new_col.replace(/\)/i,','+alphaVal+')');
            }else{
                new_col = rgbval;
                new_col = new_col.replace(/[\d\.]+\)$/g,alphaVal+')');
            }

            return new_col;
        }
	};
});
